# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/compat/proto/api_def.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorboard.compat.proto import attr_value_pb2 as tensorboard_dot_compat_dot_proto_dot_attr__value__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&tensorboard/compat/proto/api_def.proto\x12\x0btensorboard\x1a)tensorboard/compat/proto/attr_value.proto\"\xe7\x05\n\x06\x41piDef\x12\x15\n\rgraph_op_name\x18\x01 \x01(\t\x12\x1b\n\x13\x64\x65precation_message\x18\x0c \x01(\t\x12\x1b\n\x13\x64\x65precation_version\x18\r \x01(\x05\x12\x32\n\nvisibility\x18\x02 \x01(\x0e\x32\x1e.tensorboard.ApiDef.Visibility\x12.\n\x08\x65ndpoint\x18\x03 \x03(\x0b\x32\x1c.tensorboard.ApiDef.Endpoint\x12\'\n\x06in_arg\x18\x04 \x03(\x0b\x32\x17.tensorboard.ApiDef.Arg\x12(\n\x07out_arg\x18\x05 \x03(\x0b\x32\x17.tensorboard.ApiDef.Arg\x12\x11\n\targ_order\x18\x0b \x03(\t\x12&\n\x04\x61ttr\x18\x06 \x03(\x0b\x32\x18.tensorboard.ApiDef.Attr\x12\x0f\n\x07summary\x18\x07 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x08 \x01(\t\x12\x1a\n\x12\x64\x65scription_prefix\x18\t \x01(\t\x12\x1a\n\x12\x64\x65scription_suffix\x18\n \x01(\t\x1aI\n\x08\x45ndpoint\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\ndeprecated\x18\x03 \x01(\x08\x12\x1b\n\x13\x64\x65precation_version\x18\x04 \x01(\x05\x1a;\n\x03\x41rg\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\trename_to\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x1ak\n\x04\x41ttr\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\trename_to\x18\x02 \x01(\t\x12-\n\rdefault_value\x18\x03 \x01(\x0b\x32\x16.tensorboard.AttrValue\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\"G\n\nVisibility\x12\x16\n\x12\x44\x45\x46\x41ULT_VISIBILITY\x10\x00\x12\x0b\n\x07VISIBLE\x10\x01\x12\x08\n\x04SKIP\x10\x02\x12\n\n\x06HIDDEN\x10\x03\"*\n\x07\x41piDefs\x12\x1f\n\x02op\x18\x01 \x03(\x0b\x32\x13.tensorboard.ApiDefB}\n\x18org.tensorflow.frameworkB\x0c\x41piDefProtosP\x01ZNgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/api_def_go_proto\xf8\x01\x01\x62\x06proto3')



_APIDEF = DESCRIPTOR.message_types_by_name['ApiDef']
_APIDEF_ENDPOINT = _APIDEF.nested_types_by_name['Endpoint']
_APIDEF_ARG = _APIDEF.nested_types_by_name['Arg']
_APIDEF_ATTR = _APIDEF.nested_types_by_name['Attr']
_APIDEFS = DESCRIPTOR.message_types_by_name['ApiDefs']
_APIDEF_VISIBILITY = _APIDEF.enum_types_by_name['Visibility']
ApiDef = _reflection.GeneratedProtocolMessageType('ApiDef', (_message.Message,), {

  'Endpoint' : _reflection.GeneratedProtocolMessageType('Endpoint', (_message.Message,), {
    'DESCRIPTOR' : _APIDEF_ENDPOINT,
    '__module__' : 'tensorboard.compat.proto.api_def_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.ApiDef.Endpoint)
    })
  ,

  'Arg' : _reflection.GeneratedProtocolMessageType('Arg', (_message.Message,), {
    'DESCRIPTOR' : _APIDEF_ARG,
    '__module__' : 'tensorboard.compat.proto.api_def_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.ApiDef.Arg)
    })
  ,

  'Attr' : _reflection.GeneratedProtocolMessageType('Attr', (_message.Message,), {
    'DESCRIPTOR' : _APIDEF_ATTR,
    '__module__' : 'tensorboard.compat.proto.api_def_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.ApiDef.Attr)
    })
  ,
  'DESCRIPTOR' : _APIDEF,
  '__module__' : 'tensorboard.compat.proto.api_def_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.ApiDef)
  })
_sym_db.RegisterMessage(ApiDef)
_sym_db.RegisterMessage(ApiDef.Endpoint)
_sym_db.RegisterMessage(ApiDef.Arg)
_sym_db.RegisterMessage(ApiDef.Attr)

ApiDefs = _reflection.GeneratedProtocolMessageType('ApiDefs', (_message.Message,), {
  'DESCRIPTOR' : _APIDEFS,
  '__module__' : 'tensorboard.compat.proto.api_def_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.ApiDefs)
  })
_sym_db.RegisterMessage(ApiDefs)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030org.tensorflow.frameworkB\014ApiDefProtosP\001ZNgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/api_def_go_proto\370\001\001'
  _APIDEF._serialized_start=99
  _APIDEF._serialized_end=842
  _APIDEF_ENDPOINT._serialized_start=526
  _APIDEF_ENDPOINT._serialized_end=599
  _APIDEF_ARG._serialized_start=601
  _APIDEF_ARG._serialized_end=660
  _APIDEF_ATTR._serialized_start=662
  _APIDEF_ATTR._serialized_end=769
  _APIDEF_VISIBILITY._serialized_start=771
  _APIDEF_VISIBILITY._serialized_end=842
  _APIDEFS._serialized_start=844
  _APIDEFS._serialized_end=886
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/compat/proto/resource_handle.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorboard.compat.proto import tensor_shape_pb2 as tensorboard_dot_compat_dot_proto_dot_tensor__shape__pb2
from tensorboard.compat.proto import types_pb2 as tensorboard_dot_compat_dot_proto_dot_types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.tensorboard/compat/proto/resource_handle.proto\x12\x0btensorboard\x1a+tensorboard/compat/proto/tensor_shape.proto\x1a$tensorboard/compat/proto/types.proto\"\xa8\x02\n\x13ResourceHandleProto\x12\x0e\n\x06\x64\x65vice\x18\x01 \x01(\t\x12\x11\n\tcontainer\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x11\n\thash_code\x18\x04 \x01(\x04\x12\x17\n\x0fmaybe_type_name\x18\x05 \x01(\t\x12I\n\x11\x64types_and_shapes\x18\x06 \x03(\x0b\x32..tensorboard.ResourceHandleProto.DtypeAndShape\x1a\x63\n\rDtypeAndShape\x12$\n\x05\x64type\x18\x01 \x01(\x0e\x32\x15.tensorboard.DataType\x12,\n\x05shape\x18\x02 \x01(\x0b\x32\x1d.tensorboard.TensorShapeProtoJ\x04\x08\x07\x10\x08\x42\x87\x01\n\x18org.tensorflow.frameworkB\x0eResourceHandleP\x01ZVgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/resource_handle_go_proto\xf8\x01\x01\x62\x06proto3')



_RESOURCEHANDLEPROTO = DESCRIPTOR.message_types_by_name['ResourceHandleProto']
_RESOURCEHANDLEPROTO_DTYPEANDSHAPE = _RESOURCEHANDLEPROTO.nested_types_by_name['DtypeAndShape']
ResourceHandleProto = _reflection.GeneratedProtocolMessageType('ResourceHandleProto', (_message.Message,), {

  'DtypeAndShape' : _reflection.GeneratedProtocolMessageType('DtypeAndShape', (_message.Message,), {
    'DESCRIPTOR' : _RESOURCEHANDLEPROTO_DTYPEANDSHAPE,
    '__module__' : 'tensorboard.compat.proto.resource_handle_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.ResourceHandleProto.DtypeAndShape)
    })
  ,
  'DESCRIPTOR' : _RESOURCEHANDLEPROTO,
  '__module__' : 'tensorboard.compat.proto.resource_handle_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.ResourceHandleProto)
  })
_sym_db.RegisterMessage(ResourceHandleProto)
_sym_db.RegisterMessage(ResourceHandleProto.DtypeAndShape)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030org.tensorflow.frameworkB\016ResourceHandleP\001ZVgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/resource_handle_go_proto\370\001\001'
  _RESOURCEHANDLEPROTO._serialized_start=147
  _RESOURCEHANDLEPROTO._serialized_end=443
  _RESOURCEHANDLEPROTO_DTYPEANDSHAPE._serialized_start=338
  _RESOURCEHANDLEPROTO_DTYPEANDSHAPE._serialized_end=437
# @@protoc_insertion_point(module_scope)



# Copyright 2017 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""The TensorBoard Scalars plugin.

See `http_api.md` in this directory for specifications of the routes for
this plugin.
"""


import csv
import io

import werkzeug.exceptions
from werkzeug import wrappers

from tensorboard import errors
from tensorboard import plugin_util
from tensorboard.backend import http_util
from tensorboard.data import provider
from tensorboard.plugins import base_plugin
from tensorboard.plugins.scalar import metadata

_DEFAULT_DOWNSAMPLING = 1000  # scalars per time series


class OutputFormat:
    """An enum used to list the valid output formats for API calls."""

    JSON = "json"
    CSV = "csv"


class ScalarsPlugin(base_plugin.TBPlugin):
    """Scalars Plugin for TensorBoard."""

    plugin_name = metadata.PLUGIN_NAME

    def __init__(self, context):
        """Instantiates ScalarsPlugin via TensorBoard core.

        Args:
          context: A base_plugin.TBContext instance.
        """
        self._downsample_to = (context.sampling_hints or {}).get(
            self.plugin_name, _DEFAULT_DOWNSAMPLING
        )
        self._data_provider = context.data_provider
        self._version_checker = plugin_util._MetadataVersionChecker(
            data_kind="scalar",
            latest_known_version=0,
        )

    def get_plugin_apps(self):
        return {
            "/scalars": self.scalars_route,
            "/scalars_multirun": self.scalars_multirun_route,
            "/tags": self.tags_route,
        }

    def is_active(self):
        return False  # `list_plugins` as called by TB core suffices

    def frontend_metadata(self):
        return base_plugin.FrontendMetadata(element_name="tf-scalar-dashboard")

    def index_impl(self, ctx, experiment=None):
        """Return {runName: {tagName: {displayName: ..., description:
        ...}}}."""
        mapping = self._data_provider.list_scalars(
            ctx,
            experiment_id=experiment,
            plugin_name=metadata.PLUGIN_NAME,
        )
        result = {run: {} for run in mapping}
        for (run, tag_to_content) in mapping.items():
            for (tag, metadatum) in tag_to_content.items():
                md = metadata.parse_plugin_metadata(metadatum.plugin_content)
                if not self._version_checker.ok(md.version, run, tag):
                    continue
                description = plugin_util.markdown_to_safe_html(
                    metadatum.description
                )
                result[run][tag] = {
                    "displayName": metadatum.display_name,
                    "description": description,
                }
        return result

    def scalars_impl(self, ctx, tag, run, experiment, output_format):
        """Result of the form `(body, mime_type)`."""
        all_scalars = self._data_provider.read_scalars(
            ctx,
            experiment_id=experiment,
            plugin_name=metadata.PLUGIN_NAME,
            downsample=self._downsample_to,
            run_tag_filter=provider.RunTagFilter(runs=[run], tags=[tag]),
        )
        scalars = all_scalars.get(run, {}).get(tag, None)
        if scalars is None:
            raise errors.NotFoundError(
                "No scalar data for run=%r, tag=%r" % (run, tag)
            )
        values = [(x.wall_time, x.step, x.value) for x in scalars]
        if output_format == OutputFormat.CSV:
            string_io = io.StringIO()
            writer = csv.writer(string_io)
            writer.writerow(["Wall time", "Step", "Value"])
            writer.writerows(values)
            return (string_io.getvalue(), "text/csv")
        else:
            return (values, "application/json")

    def scalars_multirun_impl(self, ctx, tag, runs, experiment):
        """Result of the form `(body, mime_type)`."""
        all_scalars = self._data_provider.read_scalars(
            ctx,
            experiment_id=experiment,
            plugin_name=metadata.PLUGIN_NAME,
            downsample=self._downsample_to,
            run_tag_filter=provider.RunTagFilter(runs=runs, tags=[tag]),
        )
        body = {
            run: [(x.wall_time, x.step, x.value) for x in run_data[tag]]
            for (run, run_data) in all_scalars.items()
        }
        return (body, "application/json")

    @wrappers.Request.application
    def tags_route(self, request):
        ctx = plugin_util.context(request.environ)
        experiment = plugin_util.experiment_id(request.environ)
        index = self.index_impl(ctx, experiment=experiment)
        return http_util.Respond(request, index, "application/json")

    @wrappers.Request.application
    def scalars_route(self, request):
        """Given a tag and single run, return array of ScalarEvents."""
        tag = request.args.get("tag")
        run = request.args.get("run")
        if tag is None or run is None:
            raise errors.InvalidArgumentError(
                "Both run and tag must be specified: tag=%r, run=%r"
                % (tag, run)
            )

        ctx = plugin_util.context(request.environ)
        experiment = plugin_util.experiment_id(request.environ)
        output_format = request.args.get("format")
        (body, mime_type) = self.scalars_impl(
            ctx, tag, run, experiment, output_format
        )
        return http_util.Respond(request, body, mime_type)

    @wrappers.Request.application
    def scalars_multirun_route(self, request):
        """Given a tag and list of runs, return dict of ScalarEvent arrays."""
        if request.method != "POST":
            raise werkzeug.exceptions.MethodNotAllowed(["POST"])
        tags = request.form.getlist("tag")
        runs = request.form.getlist("runs")
        if len(tags) != 1:
            raise errors.InvalidArgumentError(
                "tag must be specified exactly once"
            )
        tag = tags[0]

        ctx = plugin_util.context(request.environ)
        experiment = plugin_util.experiment_id(request.environ)
        (body, mime_type) = self.scalars_multirun_impl(
            ctx, tag, runs, experiment
        )
        return http_util.Respond(request, body, mime_type)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/compat/proto/tensor_description.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorboard.compat.proto import allocation_description_pb2 as tensorboard_dot_compat_dot_proto_dot_allocation__description__pb2
from tensorboard.compat.proto import tensor_shape_pb2 as tensorboard_dot_compat_dot_proto_dot_tensor__shape__pb2
from tensorboard.compat.proto import types_pb2 as tensorboard_dot_compat_dot_proto_dot_types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n1tensorboard/compat/proto/tensor_description.proto\x12\x0btensorboard\x1a\x35tensorboard/compat/proto/allocation_description.proto\x1a+tensorboard/compat/proto/tensor_shape.proto\x1a$tensorboard/compat/proto/types.proto\"\xab\x01\n\x11TensorDescription\x12$\n\x05\x64type\x18\x01 \x01(\x0e\x32\x15.tensorboard.DataType\x12,\n\x05shape\x18\x02 \x01(\x0b\x32\x1d.tensorboard.TensorShapeProto\x12\x42\n\x16\x61llocation_description\x18\x04 \x01(\x0b\x32\".tensorboard.AllocationDescriptionB\x93\x01\n\x18org.tensorflow.frameworkB\x17TensorDescriptionProtosP\x01ZYgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/tensor_description_go_proto\xf8\x01\x01\x62\x06proto3')



_TENSORDESCRIPTION = DESCRIPTOR.message_types_by_name['TensorDescription']
TensorDescription = _reflection.GeneratedProtocolMessageType('TensorDescription', (_message.Message,), {
  'DESCRIPTOR' : _TENSORDESCRIPTION,
  '__module__' : 'tensorboard.compat.proto.tensor_description_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.TensorDescription)
  })
_sym_db.RegisterMessage(TensorDescription)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030org.tensorflow.frameworkB\027TensorDescriptionProtosP\001ZYgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/tensor_description_go_proto\370\001\001'
  _TENSORDESCRIPTION._serialized_start=205
  _TENSORDESCRIPTION._serialized_end=376
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/uploader/proto/export_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tensorboard.uploader.proto import blob_pb2 as tensorboard_dot_uploader_dot_proto_dot_blob__pb2
from tensorboard.uploader.proto import experiment_pb2 as tensorboard_dot_uploader_dot_proto_dot_experiment__pb2
from tensorboard.compat.proto import summary_pb2 as tensorboard_dot_compat_dot_proto_dot_summary__pb2
try:
  tensorboard_dot_compat_dot_proto_dot_histogram__pb2 = tensorboard_dot_compat_dot_proto_dot_summary__pb2.tensorboard_dot_compat_dot_proto_dot_histogram__pb2
except AttributeError:
  tensorboard_dot_compat_dot_proto_dot_histogram__pb2 = tensorboard_dot_compat_dot_proto_dot_summary__pb2.tensorboard.compat.proto.histogram_pb2
from tensorboard.compat.proto import tensor_pb2 as tensorboard_dot_compat_dot_proto_dot_tensor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n/tensorboard/uploader/proto/export_service.proto\x12\x13tensorboard.service\x1a\x1fgoogle/protobuf/timestamp.proto\x1a%tensorboard/uploader/proto/blob.proto\x1a+tensorboard/uploader/proto/experiment.proto\x1a&tensorboard/compat/proto/summary.proto\x1a%tensorboard/compat/proto/tensor.proto\"\xad\x01\n\x18StreamExperimentsRequest\x12\x32\n\x0eread_timestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0f\n\x07user_id\x18\x02 \x01(\t\x12\r\n\x05limit\x18\x03 \x01(\x03\x12=\n\x10\x65xperiments_mask\x18\x04 \x01(\x0b\x32#.tensorboard.service.ExperimentMask\"i\n\x19StreamExperimentsResponse\x12\x16\n\x0e\x65xperiment_ids\x18\x01 \x03(\t\x12\x34\n\x0b\x65xperiments\x18\x02 \x03(\x0b\x32\x1f.tensorboard.service.Experiment\"h\n\x1bStreamExperimentDataRequest\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x32\n\x0eread_timestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\xe4\x05\n\x1cStreamExperimentDataResponse\x12\x10\n\x08tag_name\x18\x01 \x01(\t\x12\x10\n\x08run_name\x18\x02 \x01(\t\x12\x32\n\x0ctag_metadata\x18\x03 \x01(\x0b\x32\x1c.tensorboard.SummaryMetadata\x12P\n\x06points\x18\x04 \x01(\x0b\x32>.tensorboard.service.StreamExperimentDataResponse.ScalarPointsH\x00\x12Q\n\x07tensors\x18\x05 \x01(\x0b\x32>.tensorboard.service.StreamExperimentDataResponse.TensorPointsH\x00\x12^\n\x0e\x62lob_sequences\x18\x06 \x01(\x0b\x32\x44.tensorboard.service.StreamExperimentDataResponse.BlobSequencePointsH\x00\x1a]\n\x0cScalarPoints\x12\r\n\x05steps\x18\x01 \x03(\x03\x12.\n\nwall_times\x18\x02 \x03(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0e\n\x06values\x18\x03 \x03(\x01\x1aw\n\x0cTensorPoints\x12\r\n\x05steps\x18\x01 \x03(\x03\x12.\n\nwall_times\x18\x02 \x03(\x0b\x32\x1a.google.protobuf.Timestamp\x12(\n\x06values\x18\x03 \x03(\x0b\x32\x18.tensorboard.TensorProto\x1a\x86\x01\n\x12\x42lobSequencePoints\x12\r\n\x05steps\x18\x01 \x03(\x03\x12.\n\nwall_times\x18\x02 \x03(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x31\n\x06values\x18\x03 \x03(\x0b\x32!.tensorboard.service.BlobSequenceB\x06\n\x04\x64\x61ta\"(\n\x15StreamBlobDataRequest\x12\x0f\n\x07\x62lob_id\x18\x01 \x01(\t\"g\n\x16StreamBlobDataResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12\x0e\n\x06offset\x18\x02 \x01(\x03\x12\x13\n\x0b\x66inal_chunk\x18\x04 \x01(\x08\x12\x14\n\x0c\x66inal_crc32c\x18\x05 \x01(\x07J\x04\x08\x03\x10\x04\x32\x84\x03\n\x1aTensorBoardExporterService\x12v\n\x11StreamExperiments\x12-.tensorboard.service.StreamExperimentsRequest\x1a..tensorboard.service.StreamExperimentsResponse\"\x00\x30\x01\x12\x7f\n\x14StreamExperimentData\x12\x30.tensorboard.service.StreamExperimentDataRequest\x1a\x31.tensorboard.service.StreamExperimentDataResponse\"\x00\x30\x01\x12m\n\x0eStreamBlobData\x12*.tensorboard.service.StreamBlobDataRequest\x1a+.tensorboard.service.StreamBlobDataResponse\"\x00\x30\x01\x62\x06proto3')



_STREAMEXPERIMENTSREQUEST = DESCRIPTOR.message_types_by_name['StreamExperimentsRequest']
_STREAMEXPERIMENTSRESPONSE = DESCRIPTOR.message_types_by_name['StreamExperimentsResponse']
_STREAMEXPERIMENTDATAREQUEST = DESCRIPTOR.message_types_by_name['StreamExperimentDataRequest']
_STREAMEXPERIMENTDATARESPONSE = DESCRIPTOR.message_types_by_name['StreamExperimentDataResponse']
_STREAMEXPERIMENTDATARESPONSE_SCALARPOINTS = _STREAMEXPERIMENTDATARESPONSE.nested_types_by_name['ScalarPoints']
_STREAMEXPERIMENTDATARESPONSE_TENSORPOINTS = _STREAMEXPERIMENTDATARESPONSE.nested_types_by_name['TensorPoints']
_STREAMEXPERIMENTDATARESPONSE_BLOBSEQUENCEPOINTS = _STREAMEXPERIMENTDATARESPONSE.nested_types_by_name['BlobSequencePoints']
_STREAMBLOBDATAREQUEST = DESCRIPTOR.message_types_by_name['StreamBlobDataRequest']
_STREAMBLOBDATARESPONSE = DESCRIPTOR.message_types_by_name['StreamBlobDataResponse']
StreamExperimentsRequest = _reflection.GeneratedProtocolMessageType('StreamExperimentsRequest', (_message.Message,), {
  'DESCRIPTOR' : _STREAMEXPERIMENTSREQUEST,
  '__module__' : 'tensorboard.uploader.proto.export_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.StreamExperimentsRequest)
  })
_sym_db.RegisterMessage(StreamExperimentsRequest)

StreamExperimentsResponse = _reflection.GeneratedProtocolMessageType('StreamExperimentsResponse', (_message.Message,), {
  'DESCRIPTOR' : _STREAMEXPERIMENTSRESPONSE,
  '__module__' : 'tensorboard.uploader.proto.export_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.StreamExperimentsResponse)
  })
_sym_db.RegisterMessage(StreamExperimentsResponse)

StreamExperimentDataRequest = _reflection.GeneratedProtocolMessageType('StreamExperimentDataRequest', (_message.Message,), {
  'DESCRIPTOR' : _STREAMEXPERIMENTDATAREQUEST,
  '__module__' : 'tensorboard.uploader.proto.export_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.StreamExperimentDataRequest)
  })
_sym_db.RegisterMessage(StreamExperimentDataRequest)

StreamExperimentDataResponse = _reflection.GeneratedProtocolMessageType('StreamExperimentDataResponse', (_message.Message,), {

  'ScalarPoints' : _reflection.GeneratedProtocolMessageType('ScalarPoints', (_message.Message,), {
    'DESCRIPTOR' : _STREAMEXPERIMENTDATARESPONSE_SCALARPOINTS,
    '__module__' : 'tensorboard.uploader.proto.export_service_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.service.StreamExperimentDataResponse.ScalarPoints)
    })
  ,

  'TensorPoints' : _reflection.GeneratedProtocolMessageType('TensorPoints', (_message.Message,), {
    'DESCRIPTOR' : _STREAMEXPERIMENTDATARESPONSE_TENSORPOINTS,
    '__module__' : 'tensorboard.uploader.proto.export_service_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.service.StreamExperimentDataResponse.TensorPoints)
    })
  ,

  'BlobSequencePoints' : _reflection.GeneratedProtocolMessageType('BlobSequencePoints', (_message.Message,), {
    'DESCRIPTOR' : _STREAMEXPERIMENTDATARESPONSE_BLOBSEQUENCEPOINTS,
    '__module__' : 'tensorboard.uploader.proto.export_service_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.service.StreamExperimentDataResponse.BlobSequencePoints)
    })
  ,
  'DESCRIPTOR' : _STREAMEXPERIMENTDATARESPONSE,
  '__module__' : 'tensorboard.uploader.proto.export_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.StreamExperimentDataResponse)
  })
_sym_db.RegisterMessage(StreamExperimentDataResponse)
_sym_db.RegisterMessage(StreamExperimentDataResponse.ScalarPoints)
_sym_db.RegisterMessage(StreamExperimentDataResponse.TensorPoints)
_sym_db.RegisterMessage(StreamExperimentDataResponse.BlobSequencePoints)

StreamBlobDataRequest = _reflection.GeneratedProtocolMessageType('StreamBlobDataRequest', (_message.Message,), {
  'DESCRIPTOR' : _STREAMBLOBDATAREQUEST,
  '__module__' : 'tensorboard.uploader.proto.export_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.StreamBlobDataRequest)
  })
_sym_db.RegisterMessage(StreamBlobDataRequest)

StreamBlobDataResponse = _reflection.GeneratedProtocolMessageType('StreamBlobDataResponse', (_message.Message,), {
  'DESCRIPTOR' : _STREAMBLOBDATARESPONSE,
  '__module__' : 'tensorboard.uploader.proto.export_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.StreamBlobDataResponse)
  })
_sym_db.RegisterMessage(StreamBlobDataResponse)

_TENSORBOARDEXPORTERSERVICE = DESCRIPTOR.services_by_name['TensorBoardExporterService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _STREAMEXPERIMENTSREQUEST._serialized_start=269
  _STREAMEXPERIMENTSREQUEST._serialized_end=442
  _STREAMEXPERIMENTSRESPONSE._serialized_start=444
  _STREAMEXPERIMENTSRESPONSE._serialized_end=549
  _STREAMEXPERIMENTDATAREQUEST._serialized_start=551
  _STREAMEXPERIMENTDATAREQUEST._serialized_end=655
  _STREAMEXPERIMENTDATARESPONSE._serialized_start=658
  _STREAMEXPERIMENTDATARESPONSE._serialized_end=1398
  _STREAMEXPERIMENTDATARESPONSE_SCALARPOINTS._serialized_start=1039
  _STREAMEXPERIMENTDATARESPONSE_SCALARPOINTS._serialized_end=1132
  _STREAMEXPERIMENTDATARESPONSE_TENSORPOINTS._serialized_start=1134
  _STREAMEXPERIMENTDATARESPONSE_TENSORPOINTS._serialized_end=1253
  _STREAMEXPERIMENTDATARESPONSE_BLOBSEQUENCEPOINTS._serialized_start=1256
  _STREAMEXPERIMENTDATARESPONSE_BLOBSEQUENCEPOINTS._serialized_end=1390
  _STREAMBLOBDATAREQUEST._serialized_start=1400
  _STREAMBLOBDATAREQUEST._serialized_end=1440
  _STREAMBLOBDATARESPONSE._serialized_start=1442
  _STREAMBLOBDATARESPONSE._serialized_end=1545
  _TENSORBOARDEXPORTERSERVICE._serialized_start=1548
  _TENSORBOARDEXPORTERSERVICE._serialized_end=1936
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/plugins/custom_scalar/layout.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.tensorboard/plugins/custom_scalar/layout.proto\x12\x0btensorboard\"\x8d\x01\n\x05\x43hart\x12\r\n\x05title\x18\x01 \x01(\t\x12\x37\n\tmultiline\x18\x02 \x01(\x0b\x32\".tensorboard.MultilineChartContentH\x00\x12\x31\n\x06margin\x18\x03 \x01(\x0b\x32\x1f.tensorboard.MarginChartContentH\x00\x42\t\n\x07\x63ontent\"$\n\x15MultilineChartContent\x12\x0b\n\x03tag\x18\x01 \x03(\t\"\x83\x01\n\x12MarginChartContent\x12\x36\n\x06series\x18\x01 \x03(\x0b\x32&.tensorboard.MarginChartContent.Series\x1a\x35\n\x06Series\x12\r\n\x05value\x18\x01 \x01(\t\x12\r\n\x05lower\x18\x02 \x01(\t\x12\r\n\x05upper\x18\x03 \x01(\t\"L\n\x08\x43\x61tegory\x12\r\n\x05title\x18\x01 \x01(\t\x12!\n\x05\x63hart\x18\x02 \x03(\x0b\x32\x12.tensorboard.Chart\x12\x0e\n\x06\x63losed\x18\x03 \x01(\x08\"B\n\x06Layout\x12\x0f\n\x07version\x18\x01 \x01(\x05\x12\'\n\x08\x63\x61tegory\x18\x02 \x03(\x0b\x32\x15.tensorboard.Categoryb\x06proto3')



_CHART = DESCRIPTOR.message_types_by_name['Chart']
_MULTILINECHARTCONTENT = DESCRIPTOR.message_types_by_name['MultilineChartContent']
_MARGINCHARTCONTENT = DESCRIPTOR.message_types_by_name['MarginChartContent']
_MARGINCHARTCONTENT_SERIES = _MARGINCHARTCONTENT.nested_types_by_name['Series']
_CATEGORY = DESCRIPTOR.message_types_by_name['Category']
_LAYOUT = DESCRIPTOR.message_types_by_name['Layout']
Chart = _reflection.GeneratedProtocolMessageType('Chart', (_message.Message,), {
  'DESCRIPTOR' : _CHART,
  '__module__' : 'tensorboard.plugins.custom_scalar.layout_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.Chart)
  })
_sym_db.RegisterMessage(Chart)

MultilineChartContent = _reflection.GeneratedProtocolMessageType('MultilineChartContent', (_message.Message,), {
  'DESCRIPTOR' : _MULTILINECHARTCONTENT,
  '__module__' : 'tensorboard.plugins.custom_scalar.layout_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.MultilineChartContent)
  })
_sym_db.RegisterMessage(MultilineChartContent)

MarginChartContent = _reflection.GeneratedProtocolMessageType('MarginChartContent', (_message.Message,), {

  'Series' : _reflection.GeneratedProtocolMessageType('Series', (_message.Message,), {
    'DESCRIPTOR' : _MARGINCHARTCONTENT_SERIES,
    '__module__' : 'tensorboard.plugins.custom_scalar.layout_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.MarginChartContent.Series)
    })
  ,
  'DESCRIPTOR' : _MARGINCHARTCONTENT,
  '__module__' : 'tensorboard.plugins.custom_scalar.layout_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.MarginChartContent)
  })
_sym_db.RegisterMessage(MarginChartContent)
_sym_db.RegisterMessage(MarginChartContent.Series)

Category = _reflection.GeneratedProtocolMessageType('Category', (_message.Message,), {
  'DESCRIPTOR' : _CATEGORY,
  '__module__' : 'tensorboard.plugins.custom_scalar.layout_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.Category)
  })
_sym_db.RegisterMessage(Category)

Layout = _reflection.GeneratedProtocolMessageType('Layout', (_message.Message,), {
  'DESCRIPTOR' : _LAYOUT,
  '__module__' : 'tensorboard.plugins.custom_scalar.layout_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.Layout)
  })
_sym_db.RegisterMessage(Layout)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _CHART._serialized_start=64
  _CHART._serialized_end=205
  _MULTILINECHARTCONTENT._serialized_start=207
  _MULTILINECHARTCONTENT._serialized_end=243
  _MARGINCHARTCONTENT._serialized_start=246
  _MARGINCHARTCONTENT._serialized_end=377
  _MARGINCHARTCONTENT_SERIES._serialized_start=324
  _MARGINCHARTCONTENT_SERIES._serialized_end=377
  _CATEGORY._serialized_start=379
  _CATEGORY._serialized_end=455
  _LAYOUT._serialized_start=457
  _LAYOUT._serialized_end=523
# @@protoc_insertion_point(module_scope)



# Copyright 2019 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Utilities for use by the uploader command line tool."""


import datetime
import errno
import os
import os.path
import time


class RateLimiter:
    """Helper class for rate-limiting using a fixed minimum interval."""

    def __init__(self, interval_secs):
        """Constructs a RateLimiter that permits a tick() every
        `interval_secs`."""
        self._time = time  # Use property for ease of testing.
        self._interval_secs = interval_secs
        self._last_called_secs = 0

    def tick(self):
        """Blocks until it has been at least `interval_secs` since last
        tick()."""
        wait_secs = (
            self._last_called_secs + self._interval_secs - self._time.time()
        )
        if wait_secs > 0:
            self._time.sleep(wait_secs)
        self._last_called_secs = self._time.time()


def get_user_config_directory():
    """Returns a platform-specific root directory for user config settings."""
    # On Windows, prefer %LOCALAPPDATA%, then %APPDATA%, since we can expect the
    # AppData directories to be ACLed to be visible only to the user and admin
    # users (https://stackoverflow.com/a/7617601/1179226). If neither is set,
    # return None instead of falling back to something that may be world-readable.
    if os.name == "nt":
        appdata = os.getenv("LOCALAPPDATA")
        if appdata:
            return appdata
        appdata = os.getenv("APPDATA")
        if appdata:
            return appdata
        return None
    # On non-windows, use XDG_CONFIG_HOME if set, else default to ~/.config.
    xdg_config_home = os.getenv("XDG_CONFIG_HOME")
    if xdg_config_home:
        return xdg_config_home
    return os.path.join(os.path.expanduser("~"), ".config")


def make_file_with_directories(path, private=False):
    """Creates a file and its containing directories, if they don't already
    exist.

    If `private` is True, the file will be made private (readable only by the
    current user) and so will the leaf directory. Pre-existing contents of the
    file are not modified.

    Passing `private=True` is not supported on Windows because it doesn't support
    the relevant parts of `os.chmod()`.

    Args:
      path: str, The path of the file to create.
      private: boolean, Whether to make the file and leaf directory readable only
        by the current user.

    Raises:
      RuntimeError: If called on Windows with `private` set to True.
    """
    if private and os.name == "nt":
        raise RuntimeError("Creating private file not supported on Windows")
    try:
        path = os.path.realpath(path)
        leaf_dir = os.path.dirname(path)
        try:
            os.makedirs(leaf_dir)
        except OSError as e:
            if e.errno != errno.EEXIST:
                raise
        if private:
            os.chmod(leaf_dir, 0o700)
        open(path, "a").close()
        if private:
            os.chmod(path, 0o600)
    except EnvironmentError as e:
        raise RuntimeError("Failed to create file %s: %s" % (path, e))


def set_timestamp(pb, seconds_since_epoch):
    """Sets a `Timestamp` proto message to a floating point UNIX time.

    This is like `pb.FromNanoseconds(int(seconds_since_epoch * 1e9))` but
    without introducing floating-point error.

    Args:
      pb: A `google.protobuf.Timestamp` message to mutate.
      seconds_since_epoch: A `float`, as returned by `time.time`.
    """
    pb.seconds = int(seconds_since_epoch)
    pb.nanos = int(round((seconds_since_epoch % 1) * 10**9))


def format_time(timestamp_pb, now=None):
    """Converts a `timestamp_pb2.Timestamp` to human-readable string.

    This always includes the absolute date and time, and for recent dates
    may include a relative time like "(just now)" or "(2 hours ago)". It
    should thus be used for ephemeral values. Use `format_time_absolute`
    if the output will be persisted.

    Args:
      timestamp_pb: A `google.protobuf.timestamp_pb2.Timestamp` value to
        convert to string. The input will not be modified.
      now: A `datetime.datetime` object representing the current time,
        used for determining relative times like "just now". Optional;
        defaults to `datetime.datetime.now()`.

    Returns:
      A string suitable for human consumption.
    """

    # Add and subtract a day for <https://bugs.python.org/issue29097>,
    # which breaks early datetime conversions on Windows for small
    # timestamps.
    dt = datetime.datetime.fromtimestamp(timestamp_pb.seconds + 86400)
    dt = dt - datetime.timedelta(seconds=86400)

    if now is None:
        now = datetime.datetime.now()
    ago = now.replace(microsecond=0) - dt

    def ago_text(n, singular, plural):
        return "%d %s ago" % (n, singular if n == 1 else plural)

    relative = None
    if ago < datetime.timedelta(seconds=5):
        relative = "just now"
    elif ago < datetime.timedelta(minutes=1):
        relative = ago_text(int(ago.total_seconds()), "second", "seconds")
    elif ago < datetime.timedelta(hours=1):
        relative = ago_text(int(ago.total_seconds()) // 60, "minute", "minutes")
    elif ago < datetime.timedelta(days=1):
        relative = ago_text(int(ago.total_seconds()) // 3600, "hour", "hours")

    relative_part = " (%s)" % relative if relative is not None else ""
    return str(dt) + relative_part


def format_time_absolute(timestamp_pb):
    """Converts a `timestamp_pb2.Timestamp` to UTC time string.

    This will always be of the form "2001-02-03T04:05:06Z".

    Args:
      timestamp_pb: A `google.protobuf.timestamp_pb2.Timestamp` value to
        convert to string. The input will not be modified.

    Returns:
      An RFC 3339 date-time string.
    """
    dt = datetime.datetime.fromtimestamp(
        timestamp_pb.seconds, tz=datetime.timezone.utc
    )
    return dt.strftime("%Y-%m-%dT%H:%M:%SZ")


def _ngettext(n, singular, plural):
    return "%d %s ago" % (n, singular if n == 1 else plural)



# Copyright 2019 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================

import struct
from tensorboard.compat.tensorflow_stub.pywrap_tensorflow import masked_crc32c


class RecordWriter:
    """Write encoded protobuf to a file with packing defined in tensorflow."""

    def __init__(self, writer):
        """Open a file to keep the tensorboard records.

        Args:
        writer: A file-like object that implements `write`, `flush` and `close`.
        """
        self._writer = writer

    # Format of a single record: (little-endian)
    # uint64    length
    # uint32    masked crc of length
    # byte      data[length]
    # uint32    masked crc of data
    def write(self, data):
        header = struct.pack("<Q", len(data))
        header_crc = struct.pack("<I", masked_crc32c(header))
        footer_crc = struct.pack("<I", masked_crc32c(data))
        self._writer.write(header + header_crc + data + footer_crc)

    def flush(self):
        self._writer.flush()

    def close(self):
        self._writer.close()

    @property
    def closed(self):
        return self._writer.closed



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/compat/proto/function.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorboard.compat.proto import attr_value_pb2 as tensorboard_dot_compat_dot_proto_dot_attr__value__pb2
from tensorboard.compat.proto import node_def_pb2 as tensorboard_dot_compat_dot_proto_dot_node__def__pb2
from tensorboard.compat.proto import op_def_pb2 as tensorboard_dot_compat_dot_proto_dot_op__def__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'tensorboard/compat/proto/function.proto\x12\x0btensorboard\x1a)tensorboard/compat/proto/attr_value.proto\x1a\'tensorboard/compat/proto/node_def.proto\x1a%tensorboard/compat/proto/op_def.proto\"\xab\x01\n\x12\x46unctionDefLibrary\x12*\n\x08\x66unction\x18\x01 \x03(\x0b\x32\x18.tensorboard.FunctionDef\x12*\n\x08gradient\x18\x02 \x03(\x0b\x32\x18.tensorboard.GradientDef\x12=\n\x14registered_gradients\x18\x03 \x03(\x0b\x32\x1f.tensorboard.RegisteredGradient\"\xcf\x06\n\x0b\x46unctionDef\x12%\n\tsignature\x18\x01 \x01(\x0b\x32\x12.tensorboard.OpDef\x12\x30\n\x04\x61ttr\x18\x05 \x03(\x0b\x32\".tensorboard.FunctionDef.AttrEntry\x12\x37\n\x08\x61rg_attr\x18\x07 \x03(\x0b\x32%.tensorboard.FunctionDef.ArgAttrEntry\x12Q\n\x16resource_arg_unique_id\x18\x08 \x03(\x0b\x32\x31.tensorboard.FunctionDef.ResourceArgUniqueIdEntry\x12&\n\x08node_def\x18\x03 \x03(\x0b\x32\x14.tensorboard.NodeDef\x12.\n\x03ret\x18\x04 \x03(\x0b\x32!.tensorboard.FunctionDef.RetEntry\x12=\n\x0b\x63ontrol_ret\x18\x06 \x03(\x0b\x32(.tensorboard.FunctionDef.ControlRetEntry\x1a\x43\n\tAttrEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12%\n\x05value\x18\x02 \x01(\x0b\x32\x16.tensorboard.AttrValue:\x02\x38\x01\x1a\x8a\x01\n\x08\x41rgAttrs\x12\x39\n\x04\x61ttr\x18\x01 \x03(\x0b\x32+.tensorboard.FunctionDef.ArgAttrs.AttrEntry\x1a\x43\n\tAttrEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12%\n\x05value\x18\x02 \x01(\x0b\x32\x16.tensorboard.AttrValue:\x02\x38\x01\x1aQ\n\x0c\x41rgAttrEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12\x30\n\x05value\x18\x02 \x01(\x0b\x32!.tensorboard.FunctionDef.ArgAttrs:\x02\x38\x01\x1a:\n\x18ResourceArgUniqueIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12\r\n\x05value\x18\x02 \x01(\r:\x02\x38\x01\x1a*\n\x08RetEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x31\n\x0f\x43ontrolRetEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01J\x04\x08\x02\x10\x03\";\n\x0bGradientDef\x12\x15\n\rfunction_name\x18\x01 \x01(\t\x12\x15\n\rgradient_func\x18\x02 \x01(\t\"G\n\x12RegisteredGradient\x12\x15\n\rgradient_func\x18\x01 \x01(\t\x12\x1a\n\x12registered_op_type\x18\x02 \x01(\tB\x80\x01\n\x18org.tensorflow.frameworkB\x0e\x46unctionProtosP\x01ZOgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/function_go_proto\xf8\x01\x01\x62\x06proto3')



_FUNCTIONDEFLIBRARY = DESCRIPTOR.message_types_by_name['FunctionDefLibrary']
_FUNCTIONDEF = DESCRIPTOR.message_types_by_name['FunctionDef']
_FUNCTIONDEF_ATTRENTRY = _FUNCTIONDEF.nested_types_by_name['AttrEntry']
_FUNCTIONDEF_ARGATTRS = _FUNCTIONDEF.nested_types_by_name['ArgAttrs']
_FUNCTIONDEF_ARGATTRS_ATTRENTRY = _FUNCTIONDEF_ARGATTRS.nested_types_by_name['AttrEntry']
_FUNCTIONDEF_ARGATTRENTRY = _FUNCTIONDEF.nested_types_by_name['ArgAttrEntry']
_FUNCTIONDEF_RESOURCEARGUNIQUEIDENTRY = _FUNCTIONDEF.nested_types_by_name['ResourceArgUniqueIdEntry']
_FUNCTIONDEF_RETENTRY = _FUNCTIONDEF.nested_types_by_name['RetEntry']
_FUNCTIONDEF_CONTROLRETENTRY = _FUNCTIONDEF.nested_types_by_name['ControlRetEntry']
_GRADIENTDEF = DESCRIPTOR.message_types_by_name['GradientDef']
_REGISTEREDGRADIENT = DESCRIPTOR.message_types_by_name['RegisteredGradient']
FunctionDefLibrary = _reflection.GeneratedProtocolMessageType('FunctionDefLibrary', (_message.Message,), {
  'DESCRIPTOR' : _FUNCTIONDEFLIBRARY,
  '__module__' : 'tensorboard.compat.proto.function_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.FunctionDefLibrary)
  })
_sym_db.RegisterMessage(FunctionDefLibrary)

FunctionDef = _reflection.GeneratedProtocolMessageType('FunctionDef', (_message.Message,), {

  'AttrEntry' : _reflection.GeneratedProtocolMessageType('AttrEntry', (_message.Message,), {
    'DESCRIPTOR' : _FUNCTIONDEF_ATTRENTRY,
    '__module__' : 'tensorboard.compat.proto.function_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.FunctionDef.AttrEntry)
    })
  ,

  'ArgAttrs' : _reflection.GeneratedProtocolMessageType('ArgAttrs', (_message.Message,), {

    'AttrEntry' : _reflection.GeneratedProtocolMessageType('AttrEntry', (_message.Message,), {
      'DESCRIPTOR' : _FUNCTIONDEF_ARGATTRS_ATTRENTRY,
      '__module__' : 'tensorboard.compat.proto.function_pb2'
      # @@protoc_insertion_point(class_scope:tensorboard.FunctionDef.ArgAttrs.AttrEntry)
      })
    ,
    'DESCRIPTOR' : _FUNCTIONDEF_ARGATTRS,
    '__module__' : 'tensorboard.compat.proto.function_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.FunctionDef.ArgAttrs)
    })
  ,

  'ArgAttrEntry' : _reflection.GeneratedProtocolMessageType('ArgAttrEntry', (_message.Message,), {
    'DESCRIPTOR' : _FUNCTIONDEF_ARGATTRENTRY,
    '__module__' : 'tensorboard.compat.proto.function_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.FunctionDef.ArgAttrEntry)
    })
  ,

  'ResourceArgUniqueIdEntry' : _reflection.GeneratedProtocolMessageType('ResourceArgUniqueIdEntry', (_message.Message,), {
    'DESCRIPTOR' : _FUNCTIONDEF_RESOURCEARGUNIQUEIDENTRY,
    '__module__' : 'tensorboard.compat.proto.function_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.FunctionDef.ResourceArgUniqueIdEntry)
    })
  ,

  'RetEntry' : _reflection.GeneratedProtocolMessageType('RetEntry', (_message.Message,), {
    'DESCRIPTOR' : _FUNCTIONDEF_RETENTRY,
    '__module__' : 'tensorboard.compat.proto.function_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.FunctionDef.RetEntry)
    })
  ,

  'ControlRetEntry' : _reflection.GeneratedProtocolMessageType('ControlRetEntry', (_message.Message,), {
    'DESCRIPTOR' : _FUNCTIONDEF_CONTROLRETENTRY,
    '__module__' : 'tensorboard.compat.proto.function_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.FunctionDef.ControlRetEntry)
    })
  ,
  'DESCRIPTOR' : _FUNCTIONDEF,
  '__module__' : 'tensorboard.compat.proto.function_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.FunctionDef)
  })
_sym_db.RegisterMessage(FunctionDef)
_sym_db.RegisterMessage(FunctionDef.AttrEntry)
_sym_db.RegisterMessage(FunctionDef.ArgAttrs)
_sym_db.RegisterMessage(FunctionDef.ArgAttrs.AttrEntry)
_sym_db.RegisterMessage(FunctionDef.ArgAttrEntry)
_sym_db.RegisterMessage(FunctionDef.ResourceArgUniqueIdEntry)
_sym_db.RegisterMessage(FunctionDef.RetEntry)
_sym_db.RegisterMessage(FunctionDef.ControlRetEntry)

GradientDef = _reflection.GeneratedProtocolMessageType('GradientDef', (_message.Message,), {
  'DESCRIPTOR' : _GRADIENTDEF,
  '__module__' : 'tensorboard.compat.proto.function_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.GradientDef)
  })
_sym_db.RegisterMessage(GradientDef)

RegisteredGradient = _reflection.GeneratedProtocolMessageType('RegisteredGradient', (_message.Message,), {
  'DESCRIPTOR' : _REGISTEREDGRADIENT,
  '__module__' : 'tensorboard.compat.proto.function_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.RegisteredGradient)
  })
_sym_db.RegisterMessage(RegisteredGradient)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030org.tensorflow.frameworkB\016FunctionProtosP\001ZOgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/function_go_proto\370\001\001'
  _FUNCTIONDEF_ATTRENTRY._options = None
  _FUNCTIONDEF_ATTRENTRY._serialized_options = b'8\001'
  _FUNCTIONDEF_ARGATTRS_ATTRENTRY._options = None
  _FUNCTIONDEF_ARGATTRS_ATTRENTRY._serialized_options = b'8\001'
  _FUNCTIONDEF_ARGATTRENTRY._options = None
  _FUNCTIONDEF_ARGATTRENTRY._serialized_options = b'8\001'
  _FUNCTIONDEF_RESOURCEARGUNIQUEIDENTRY._options = None
  _FUNCTIONDEF_RESOURCEARGUNIQUEIDENTRY._serialized_options = b'8\001'
  _FUNCTIONDEF_RETENTRY._options = None
  _FUNCTIONDEF_RETENTRY._serialized_options = b'8\001'
  _FUNCTIONDEF_CONTROLRETENTRY._options = None
  _FUNCTIONDEF_CONTROLRETENTRY._serialized_options = b'8\001'
  _FUNCTIONDEFLIBRARY._serialized_start=180
  _FUNCTIONDEFLIBRARY._serialized_end=351
  _FUNCTIONDEF._serialized_start=354
  _FUNCTIONDEF._serialized_end=1201
  _FUNCTIONDEF_ATTRENTRY._serialized_start=749
  _FUNCTIONDEF_ATTRENTRY._serialized_end=816
  _FUNCTIONDEF_ARGATTRS._serialized_start=819
  _FUNCTIONDEF_ARGATTRS._serialized_end=957
  _FUNCTIONDEF_ARGATTRS_ATTRENTRY._serialized_start=749
  _FUNCTIONDEF_ARGATTRS_ATTRENTRY._serialized_end=816
  _FUNCTIONDEF_ARGATTRENTRY._serialized_start=959
  _FUNCTIONDEF_ARGATTRENTRY._serialized_end=1040
  _FUNCTIONDEF_RESOURCEARGUNIQUEIDENTRY._serialized_start=1042
  _FUNCTIONDEF_RESOURCEARGUNIQUEIDENTRY._serialized_end=1100
  _FUNCTIONDEF_RETENTRY._serialized_start=1102
  _FUNCTIONDEF_RETENTRY._serialized_end=1144
  _FUNCTIONDEF_CONTROLRETENTRY._serialized_start=1146
  _FUNCTIONDEF_CONTROLRETENTRY._serialized_end=1195
  _GRADIENTDEF._serialized_start=1203
  _GRADIENTDEF._serialized_end=1262
  _REGISTEREDGRADIENT._serialized_start=1264
  _REGISTEREDGRADIENT._serialized_end=1335
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/plugins/mesh/plugin_data.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*tensorboard/plugins/mesh/plugin_data.proto\x12\x10tensorboard.mesh\"\xea\x01\n\x0eMeshPluginData\x12\x0f\n\x07version\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x42\n\x0c\x63ontent_type\x18\x03 \x01(\x0e\x32,.tensorboard.mesh.MeshPluginData.ContentType\x12\x13\n\x0bjson_config\x18\x05 \x01(\t\x12\r\n\x05shape\x18\x06 \x03(\x05\x12\x12\n\ncomponents\x18\x07 \x01(\r\"=\n\x0b\x43ontentType\x12\r\n\tUNDEFINED\x10\x00\x12\n\n\x06VERTEX\x10\x01\x12\x08\n\x04\x46\x41\x43\x45\x10\x02\x12\t\n\x05\x43OLOR\x10\x03\x62\x06proto3')



_MESHPLUGINDATA = DESCRIPTOR.message_types_by_name['MeshPluginData']
_MESHPLUGINDATA_CONTENTTYPE = _MESHPLUGINDATA.enum_types_by_name['ContentType']
MeshPluginData = _reflection.GeneratedProtocolMessageType('MeshPluginData', (_message.Message,), {
  'DESCRIPTOR' : _MESHPLUGINDATA,
  '__module__' : 'tensorboard.plugins.mesh.plugin_data_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.mesh.MeshPluginData)
  })
_sym_db.RegisterMessage(MeshPluginData)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _MESHPLUGINDATA._serialized_start=65
  _MESHPLUGINDATA._serialized_end=299
  _MESHPLUGINDATA_CONTENTTYPE._serialized_start=238
  _MESHPLUGINDATA_CONTENTTYPE._serialized_end=299
# @@protoc_insertion_point(module_scope)



# Copyright 2018 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Text summaries and TensorFlow operations to create them, V2 versions."""


import numpy as np

from tensorboard.compat import tf2 as tf
from tensorboard.compat.proto import summary_pb2
from tensorboard.plugins.text import metadata
from tensorboard.util import tensor_util


def text(name, data, step=None, description=None):
    r"""Write a text summary.

    See also `tf.summary.scalar`, `tf.summary.SummaryWriter`, `tf.summary.image`.

    Writes text Tensor values for later visualization and analysis in TensorBoard.
    Writes go to the current default summary writer.  Like `tf.summary.scalar`
    points, text points are each associated with a `step` and a `name`.
    All the points with the same `name` constitute a time series of text values.

    For Example:
    ```python
    test_summary_writer = tf.summary.create_file_writer('test/logdir')
    with test_summary_writer.as_default():
        tf.summary.text('first_text', 'hello world!', step=0)
        tf.summary.text('first_text', 'nice to meet you!', step=1)
    ```

    The text summary can also contain Markdown, and TensorBoard will render the text
    as such.

    ```python
    with test_summary_writer.as_default():
        text_data = '''
              | *hello* | *there* |
              |---------|---------|
              | this    | is      |
              | a       | table   |
        '''
        text_data = '\n'.join(l.strip() for l in text_data.splitlines())
        tf.summary.text('markdown_text', text_data, step=0)
    ```

    Since text is Tensor valued, each text point may be a Tensor of string values.
    rank-1 and rank-2 Tensors are rendered as tables in TensorBoard.  For higher ranked
    Tensors, you'll see just a 2D slice of the data.  To avoid this, reshape the Tensor
    to at most rank-2 prior to passing it to this function.

    Demo notebook at
    ["Displaying text data in TensorBoard"](https://www.tensorflow.org/tensorboard/text_summaries).

    Arguments:
      name: A name for this summary. The summary tag used for TensorBoard will
        be this name prefixed by any active name scopes.
      data: A UTF-8 string Tensor value.
      step: Explicit `int64`-castable monotonic step value for this summary. If
        omitted, this defaults to `tf.summary.experimental.get_step()`, which must
        not be None.
      description: Optional long-form description for this summary, as a
        constant `str`. Markdown is supported. Defaults to empty.

    Returns:
      True on success, or false if no summary was emitted because no default
      summary writer was available.

    Raises:
      ValueError: if a default writer exists, but no step was provided and
        `tf.summary.experimental.get_step()` is None.
    """
    summary_metadata = metadata.create_summary_metadata(
        display_name=None, description=description
    )
    # TODO(https://github.com/tensorflow/tensorboard/issues/2109): remove fallback
    summary_scope = (
        getattr(tf.summary.experimental, "summary_scope", None)
        or tf.summary.summary_scope
    )
    with summary_scope(name, "text_summary", values=[data, step]) as (tag, _):
        tf.debugging.assert_type(data, tf.string)
        return tf.summary.write(
            tag=tag, tensor=data, step=step, metadata=summary_metadata
        )


def text_pb(tag, data, description=None):
    """Create a text tf.Summary protobuf.

    Arguments:
      tag: String tag for the summary.
      data: A Python bytestring (of type bytes), a Unicode string, or a numpy data
        array of those types.
      description: Optional long-form description for this summary, as a `str`.
        Markdown is supported. Defaults to empty.

    Raises:
      TypeError: If the type of the data is unsupported.

    Returns:
      A `tf.Summary` protobuf object.
    """
    try:
        tensor = tensor_util.make_tensor_proto(data, dtype=np.object_)
    except TypeError as e:
        raise TypeError("tensor must be of type string", e)
    summary_metadata = metadata.create_summary_metadata(
        display_name=None, description=description
    )
    summary = summary_pb2.Summary()
    summary.value.add(tag=tag, metadata=summary_metadata, tensor=tensor)
    return summary



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/compat/proto/saved_object_graph.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from tensorboard.compat.proto import tensor_shape_pb2 as tensorboard_dot_compat_dot_proto_dot_tensor__shape__pb2
from tensorboard.compat.proto import types_pb2 as tensorboard_dot_compat_dot_proto_dot_types__pb2
from tensorboard.compat.proto import variable_pb2 as tensorboard_dot_compat_dot_proto_dot_variable__pb2
from tensorboard.compat.proto import versions_pb2 as tensorboard_dot_compat_dot_proto_dot_versions__pb2
from tensorboard.compat.proto import struct_pb2 as tensorboard_dot_compat_dot_proto_dot_struct__pb2
from tensorboard.compat.proto import trackable_object_graph_pb2 as tensorboard_dot_compat_dot_proto_dot_trackable__object__graph__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n1tensorboard/compat/proto/saved_object_graph.proto\x12\x0btensorboard\x1a\x19google/protobuf/any.proto\x1a+tensorboard/compat/proto/tensor_shape.proto\x1a$tensorboard/compat/proto/types.proto\x1a\'tensorboard/compat/proto/variable.proto\x1a\'tensorboard/compat/proto/versions.proto\x1a%tensorboard/compat/proto/struct.proto\x1a\x35tensorboard/compat/proto/trackable_object_graph.proto\"\xeb\x01\n\x10SavedObjectGraph\x12\'\n\x05nodes\x18\x01 \x03(\x0b\x32\x18.tensorboard.SavedObject\x12P\n\x12\x63oncrete_functions\x18\x02 \x03(\x0b\x32\x34.tensorboard.SavedObjectGraph.ConcreteFunctionsEntry\x1a\\\n\x16\x43oncreteFunctionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\".tensorboard.SavedConcreteFunction:\x02\x38\x01\"\xdd\x07\n\x0bSavedObject\x12S\n\x08\x63hildren\x18\x01 \x03(\x0b\x32\x41.tensorboard.TrackableObjectGraph.TrackableObject.ObjectReference\x12W\n\x0c\x64\x65pendencies\x18\x0f \x03(\x0b\x32\x41.tensorboard.TrackableObjectGraph.TrackableObject.ObjectReference\x12_\n\x0eslot_variables\x18\x03 \x03(\x0b\x32G.tensorboard.TrackableObjectGraph.TrackableObject.SlotVariableReference\x12\x33\n\x0buser_object\x18\x04 \x01(\x0b\x32\x1c.tensorboard.SavedUserObjectH\x00\x12(\n\x05\x61sset\x18\x05 \x01(\x0b\x32\x17.tensorboard.SavedAssetH\x00\x12.\n\x08\x66unction\x18\x06 \x01(\x0b\x32\x1a.tensorboard.SavedFunctionH\x00\x12.\n\x08variable\x18\x07 \x01(\x0b\x32\x1a.tensorboard.SavedVariableH\x00\x12H\n\x16\x62\x61re_concrete_function\x18\x08 \x01(\x0b\x32&.tensorboard.SavedBareConcreteFunctionH\x00\x12.\n\x08\x63onstant\x18\t \x01(\x0b\x32\x1a.tensorboard.SavedConstantH\x00\x12.\n\x08resource\x18\n \x01(\x0b\x32\x1a.tensorboard.SavedResourceH\x00\x12\x36\n\x0f\x63\x61ptured_tensor\x18\x0c \x01(\x0b\x32\x1b.tensorboard.CapturedTensorH\x00\x12G\n\x10saveable_objects\x18\x0b \x03(\x0b\x32-.tensorboard.SavedObject.SaveableObjectsEntry\x12\x17\n\x0fregistered_name\x18\r \x01(\t\x12\x33\n\x15serialized_user_proto\x18\x0e \x01(\x0b\x32\x14.google.protobuf.Any\x12\x18\n\x10registered_saver\x18\x10 \x01(\t\x1aS\n\x14SaveableObjectsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12*\n\x05value\x18\x02 \x01(\x0b\x32\x1b.tensorboard.SaveableObject:\x02\x38\x01\x42\x06\n\x04kindJ\x04\x08\x02\x10\x03R\nattributes\"e\n\x0fSavedUserObject\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12(\n\x07version\x18\x02 \x01(\x0b\x32\x17.tensorboard.VersionDef\x12\x14\n\x08metadata\x18\x03 \x01(\tB\x02\x18\x01\"*\n\nSavedAsset\x12\x1c\n\x14\x61sset_file_def_index\x18\x01 \x01(\x05\"]\n\rSavedFunction\x12\x1a\n\x12\x63oncrete_functions\x18\x01 \x03(\t\x12\x30\n\rfunction_spec\x18\x02 \x01(\x0b\x32\x19.tensorboard.FunctionSpec\"9\n\x0e\x43\x61pturedTensor\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x19\n\x11\x63oncrete_function\x18\x02 \x01(\t\"\xaa\x01\n\x15SavedConcreteFunction\x12\x14\n\x0c\x62ound_inputs\x18\x02 \x03(\x05\x12\x43\n\x1d\x63\x61nonicalized_input_signature\x18\x03 \x01(\x0b\x32\x1c.tensorboard.StructuredValue\x12\x36\n\x10output_signature\x18\x04 \x01(\x0b\x32\x1c.tensorboard.StructuredValue\"\xae\x01\n\x19SavedBareConcreteFunction\x12\x1e\n\x16\x63oncrete_function_name\x18\x01 \x01(\t\x12\x19\n\x11\x61rgument_keywords\x18\x02 \x03(\t\x12$\n\x1c\x61llowed_positional_arguments\x18\x03 \x01(\x03\x12\x30\n\rfunction_spec\x18\x04 \x01(\x0b\x32\x19.tensorboard.FunctionSpec\"\"\n\rSavedConstant\x12\x11\n\toperation\x18\x01 \x01(\t\"\xdc\x02\n\rSavedVariable\x12$\n\x05\x64type\x18\x01 \x01(\x0e\x32\x15.tensorboard.DataType\x12,\n\x05shape\x18\x02 \x01(\x0b\x32\x1d.tensorboard.TensorShapeProto\x12\x11\n\ttrainable\x18\x03 \x01(\x08\x12=\n\x0fsynchronization\x18\x04 \x01(\x0e\x32$.tensorboard.VariableSynchronization\x12\x35\n\x0b\x61ggregation\x18\x05 \x01(\x0e\x32 .tensorboard.VariableAggregation\x12\x0c\n\x04name\x18\x06 \x01(\t\x12\x0e\n\x06\x64\x65vice\x18\x07 \x01(\t\x12P\n,experimental_distributed_variable_components\x18\x08 \x03(\x0b\x32\x1a.tensorboard.SavedVariable\"\xfe\x01\n\x0c\x46unctionSpec\x12\x31\n\x0b\x66ullargspec\x18\x01 \x01(\x0b\x32\x1c.tensorboard.StructuredValue\x12\x11\n\tis_method\x18\x02 \x01(\x08\x12\x35\n\x0finput_signature\x18\x05 \x01(\x0b\x32\x1c.tensorboard.StructuredValue\x12\x39\n\x0bjit_compile\x18\x06 \x01(\x0e\x32$.tensorboard.FunctionSpec.JitCompile\"*\n\nJitCompile\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\x06\n\x02ON\x10\x01\x12\x07\n\x03OFF\x10\x02J\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05\"\x1f\n\rSavedResource\x12\x0e\n\x06\x64\x65vice\x18\x01 \x01(\t\"A\n\x0eSaveableObject\x12\x15\n\rsave_function\x18\x02 \x01(\x05\x12\x18\n\x10restore_function\x18\x03 \x01(\x05\x42ZZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01\x62\x06proto3')



_SAVEDOBJECTGRAPH = DESCRIPTOR.message_types_by_name['SavedObjectGraph']
_SAVEDOBJECTGRAPH_CONCRETEFUNCTIONSENTRY = _SAVEDOBJECTGRAPH.nested_types_by_name['ConcreteFunctionsEntry']
_SAVEDOBJECT = DESCRIPTOR.message_types_by_name['SavedObject']
_SAVEDOBJECT_SAVEABLEOBJECTSENTRY = _SAVEDOBJECT.nested_types_by_name['SaveableObjectsEntry']
_SAVEDUSEROBJECT = DESCRIPTOR.message_types_by_name['SavedUserObject']
_SAVEDASSET = DESCRIPTOR.message_types_by_name['SavedAsset']
_SAVEDFUNCTION = DESCRIPTOR.message_types_by_name['SavedFunction']
_CAPTUREDTENSOR = DESCRIPTOR.message_types_by_name['CapturedTensor']
_SAVEDCONCRETEFUNCTION = DESCRIPTOR.message_types_by_name['SavedConcreteFunction']
_SAVEDBARECONCRETEFUNCTION = DESCRIPTOR.message_types_by_name['SavedBareConcreteFunction']
_SAVEDCONSTANT = DESCRIPTOR.message_types_by_name['SavedConstant']
_SAVEDVARIABLE = DESCRIPTOR.message_types_by_name['SavedVariable']
_FUNCTIONSPEC = DESCRIPTOR.message_types_by_name['FunctionSpec']
_SAVEDRESOURCE = DESCRIPTOR.message_types_by_name['SavedResource']
_SAVEABLEOBJECT = DESCRIPTOR.message_types_by_name['SaveableObject']
_FUNCTIONSPEC_JITCOMPILE = _FUNCTIONSPEC.enum_types_by_name['JitCompile']
SavedObjectGraph = _reflection.GeneratedProtocolMessageType('SavedObjectGraph', (_message.Message,), {

  'ConcreteFunctionsEntry' : _reflection.GeneratedProtocolMessageType('ConcreteFunctionsEntry', (_message.Message,), {
    'DESCRIPTOR' : _SAVEDOBJECTGRAPH_CONCRETEFUNCTIONSENTRY,
    '__module__' : 'tensorboard.compat.proto.saved_object_graph_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.SavedObjectGraph.ConcreteFunctionsEntry)
    })
  ,
  'DESCRIPTOR' : _SAVEDOBJECTGRAPH,
  '__module__' : 'tensorboard.compat.proto.saved_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.SavedObjectGraph)
  })
_sym_db.RegisterMessage(SavedObjectGraph)
_sym_db.RegisterMessage(SavedObjectGraph.ConcreteFunctionsEntry)

SavedObject = _reflection.GeneratedProtocolMessageType('SavedObject', (_message.Message,), {

  'SaveableObjectsEntry' : _reflection.GeneratedProtocolMessageType('SaveableObjectsEntry', (_message.Message,), {
    'DESCRIPTOR' : _SAVEDOBJECT_SAVEABLEOBJECTSENTRY,
    '__module__' : 'tensorboard.compat.proto.saved_object_graph_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.SavedObject.SaveableObjectsEntry)
    })
  ,
  'DESCRIPTOR' : _SAVEDOBJECT,
  '__module__' : 'tensorboard.compat.proto.saved_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.SavedObject)
  })
_sym_db.RegisterMessage(SavedObject)
_sym_db.RegisterMessage(SavedObject.SaveableObjectsEntry)

SavedUserObject = _reflection.GeneratedProtocolMessageType('SavedUserObject', (_message.Message,), {
  'DESCRIPTOR' : _SAVEDUSEROBJECT,
  '__module__' : 'tensorboard.compat.proto.saved_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.SavedUserObject)
  })
_sym_db.RegisterMessage(SavedUserObject)

SavedAsset = _reflection.GeneratedProtocolMessageType('SavedAsset', (_message.Message,), {
  'DESCRIPTOR' : _SAVEDASSET,
  '__module__' : 'tensorboard.compat.proto.saved_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.SavedAsset)
  })
_sym_db.RegisterMessage(SavedAsset)

SavedFunction = _reflection.GeneratedProtocolMessageType('SavedFunction', (_message.Message,), {
  'DESCRIPTOR' : _SAVEDFUNCTION,
  '__module__' : 'tensorboard.compat.proto.saved_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.SavedFunction)
  })
_sym_db.RegisterMessage(SavedFunction)

CapturedTensor = _reflection.GeneratedProtocolMessageType('CapturedTensor', (_message.Message,), {
  'DESCRIPTOR' : _CAPTUREDTENSOR,
  '__module__' : 'tensorboard.compat.proto.saved_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.CapturedTensor)
  })
_sym_db.RegisterMessage(CapturedTensor)

SavedConcreteFunction = _reflection.GeneratedProtocolMessageType('SavedConcreteFunction', (_message.Message,), {
  'DESCRIPTOR' : _SAVEDCONCRETEFUNCTION,
  '__module__' : 'tensorboard.compat.proto.saved_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.SavedConcreteFunction)
  })
_sym_db.RegisterMessage(SavedConcreteFunction)

SavedBareConcreteFunction = _reflection.GeneratedProtocolMessageType('SavedBareConcreteFunction', (_message.Message,), {
  'DESCRIPTOR' : _SAVEDBARECONCRETEFUNCTION,
  '__module__' : 'tensorboard.compat.proto.saved_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.SavedBareConcreteFunction)
  })
_sym_db.RegisterMessage(SavedBareConcreteFunction)

SavedConstant = _reflection.GeneratedProtocolMessageType('SavedConstant', (_message.Message,), {
  'DESCRIPTOR' : _SAVEDCONSTANT,
  '__module__' : 'tensorboard.compat.proto.saved_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.SavedConstant)
  })
_sym_db.RegisterMessage(SavedConstant)

SavedVariable = _reflection.GeneratedProtocolMessageType('SavedVariable', (_message.Message,), {
  'DESCRIPTOR' : _SAVEDVARIABLE,
  '__module__' : 'tensorboard.compat.proto.saved_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.SavedVariable)
  })
_sym_db.RegisterMessage(SavedVariable)

FunctionSpec = _reflection.GeneratedProtocolMessageType('FunctionSpec', (_message.Message,), {
  'DESCRIPTOR' : _FUNCTIONSPEC,
  '__module__' : 'tensorboard.compat.proto.saved_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.FunctionSpec)
  })
_sym_db.RegisterMessage(FunctionSpec)

SavedResource = _reflection.GeneratedProtocolMessageType('SavedResource', (_message.Message,), {
  'DESCRIPTOR' : _SAVEDRESOURCE,
  '__module__' : 'tensorboard.compat.proto.saved_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.SavedResource)
  })
_sym_db.RegisterMessage(SavedResource)

SaveableObject = _reflection.GeneratedProtocolMessageType('SaveableObject', (_message.Message,), {
  'DESCRIPTOR' : _SAVEABLEOBJECT,
  '__module__' : 'tensorboard.compat.proto.saved_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.SaveableObject)
  })
_sym_db.RegisterMessage(SaveableObject)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\370\001\001'
  _SAVEDOBJECTGRAPH_CONCRETEFUNCTIONSENTRY._options = None
  _SAVEDOBJECTGRAPH_CONCRETEFUNCTIONSENTRY._serialized_options = b'8\001'
  _SAVEDOBJECT_SAVEABLEOBJECTSENTRY._options = None
  _SAVEDOBJECT_SAVEABLEOBJECTSENTRY._serialized_options = b'8\001'
  _SAVEDUSEROBJECT.fields_by_name['metadata']._options = None
  _SAVEDUSEROBJECT.fields_by_name['metadata']._serialized_options = b'\030\001'
  _SAVEDOBJECTGRAPH._serialized_start=353
  _SAVEDOBJECTGRAPH._serialized_end=588
  _SAVEDOBJECTGRAPH_CONCRETEFUNCTIONSENTRY._serialized_start=496
  _SAVEDOBJECTGRAPH_CONCRETEFUNCTIONSENTRY._serialized_end=588
  _SAVEDOBJECT._serialized_start=591
  _SAVEDOBJECT._serialized_end=1580
  _SAVEDOBJECT_SAVEABLEOBJECTSENTRY._serialized_start=1471
  _SAVEDOBJECT_SAVEABLEOBJECTSENTRY._serialized_end=1554
  _SAVEDUSEROBJECT._serialized_start=1582
  _SAVEDUSEROBJECT._serialized_end=1683
  _SAVEDASSET._serialized_start=1685
  _SAVEDASSET._serialized_end=1727
  _SAVEDFUNCTION._serialized_start=1729
  _SAVEDFUNCTION._serialized_end=1822
  _CAPTUREDTENSOR._serialized_start=1824
  _CAPTUREDTENSOR._serialized_end=1881
  _SAVEDCONCRETEFUNCTION._serialized_start=1884
  _SAVEDCONCRETEFUNCTION._serialized_end=2054
  _SAVEDBARECONCRETEFUNCTION._serialized_start=2057
  _SAVEDBARECONCRETEFUNCTION._serialized_end=2231
  _SAVEDCONSTANT._serialized_start=2233
  _SAVEDCONSTANT._serialized_end=2267
  _SAVEDVARIABLE._serialized_start=2270
  _SAVEDVARIABLE._serialized_end=2618
  _FUNCTIONSPEC._serialized_start=2621
  _FUNCTIONSPEC._serialized_end=2875
  _FUNCTIONSPEC_JITCOMPILE._serialized_start=2821
  _FUNCTIONSPEC_JITCOMPILE._serialized_end=2863
  _SAVEDRESOURCE._serialized_start=2877
  _SAVEDRESOURCE._serialized_end=2908
  _SAVEABLEOBJECT._serialized_start=2910
  _SAVEABLEOBJECT._serialized_end=2975
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from tensorboard.uploader.proto import export_service_pb2 as tensorboard_dot_uploader_dot_proto_dot_export__service__pb2


class TensorBoardExporterServiceStub(object):
    """Service for exporting data from TensorBoard.dev.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.StreamExperiments = channel.unary_stream(
                '/tensorboard.service.TensorBoardExporterService/StreamExperiments',
                request_serializer=tensorboard_dot_uploader_dot_proto_dot_export__service__pb2.StreamExperimentsRequest.SerializeToString,
                response_deserializer=tensorboard_dot_uploader_dot_proto_dot_export__service__pb2.StreamExperimentsResponse.FromString,
                )
        self.StreamExperimentData = channel.unary_stream(
                '/tensorboard.service.TensorBoardExporterService/StreamExperimentData',
                request_serializer=tensorboard_dot_uploader_dot_proto_dot_export__service__pb2.StreamExperimentDataRequest.SerializeToString,
                response_deserializer=tensorboard_dot_uploader_dot_proto_dot_export__service__pb2.StreamExperimentDataResponse.FromString,
                )
        self.StreamBlobData = channel.unary_stream(
                '/tensorboard.service.TensorBoardExporterService/StreamBlobData',
                request_serializer=tensorboard_dot_uploader_dot_proto_dot_export__service__pb2.StreamBlobDataRequest.SerializeToString,
                response_deserializer=tensorboard_dot_uploader_dot_proto_dot_export__service__pb2.StreamBlobDataResponse.FromString,
                )


class TensorBoardExporterServiceServicer(object):
    """Service for exporting data from TensorBoard.dev.
    """

    def StreamExperiments(self, request, context):
        """Stream the experiment_id of all the experiments owned by the caller.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StreamExperimentData(self, request, context):
        """Stream scalars for all the runs and tags in an experiment.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StreamBlobData(self, request, context):
        """Stream blob as chunks for a given blob_id.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_TensorBoardExporterServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'StreamExperiments': grpc.unary_stream_rpc_method_handler(
                    servicer.StreamExperiments,
                    request_deserializer=tensorboard_dot_uploader_dot_proto_dot_export__service__pb2.StreamExperimentsRequest.FromString,
                    response_serializer=tensorboard_dot_uploader_dot_proto_dot_export__service__pb2.StreamExperimentsResponse.SerializeToString,
            ),
            'StreamExperimentData': grpc.unary_stream_rpc_method_handler(
                    servicer.StreamExperimentData,
                    request_deserializer=tensorboard_dot_uploader_dot_proto_dot_export__service__pb2.StreamExperimentDataRequest.FromString,
                    response_serializer=tensorboard_dot_uploader_dot_proto_dot_export__service__pb2.StreamExperimentDataResponse.SerializeToString,
            ),
            'StreamBlobData': grpc.unary_stream_rpc_method_handler(
                    servicer.StreamBlobData,
                    request_deserializer=tensorboard_dot_uploader_dot_proto_dot_export__service__pb2.StreamBlobDataRequest.FromString,
                    response_serializer=tensorboard_dot_uploader_dot_proto_dot_export__service__pb2.StreamBlobDataResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'tensorboard.service.TensorBoardExporterService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class TensorBoardExporterService(object):
    """Service for exporting data from TensorBoard.dev.
    """

    @staticmethod
    def StreamExperiments(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/tensorboard.service.TensorBoardExporterService/StreamExperiments',
            tensorboard_dot_uploader_dot_proto_dot_export__service__pb2.StreamExperimentsRequest.SerializeToString,
            tensorboard_dot_uploader_dot_proto_dot_export__service__pb2.StreamExperimentsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StreamExperimentData(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/tensorboard.service.TensorBoardExporterService/StreamExperimentData',
            tensorboard_dot_uploader_dot_proto_dot_export__service__pb2.StreamExperimentDataRequest.SerializeToString,
            tensorboard_dot_uploader_dot_proto_dot_export__service__pb2.StreamExperimentDataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StreamBlobData(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/tensorboard.service.TensorBoardExporterService/StreamBlobData',
            tensorboard_dot_uploader_dot_proto_dot_export__service__pb2.StreamBlobDataRequest.SerializeToString,
            tensorboard_dot_uploader_dot_proto_dot_export__service__pb2.StreamBlobDataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Copyright 2023 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
from tensorboard import auth
from tensorboard import context


class AuthContextMiddleware:
    """WSGI middleware to inject an AuthContext into the RequestContext."""

    def __init__(self, application, auth_providers):
        """Initializes this middleware.

        Args:
          application: A WSGI application to delegate to.
          auth_providers: The auth_providers to provide to the AuthContext.
        """
        self._application = application
        self._auth_providers = auth_providers

    def __call__(self, environ, start_response):
        """Invoke this WSGI application."""
        environ = dict(environ)
        auth_ctx = auth.AuthContext(self._auth_providers, environ)
        ctx = context.from_environ(environ).replace(auth=auth_ctx)
        context.set_in_environ(environ, ctx)
        return self._application(environ, start_response)



# Copyright 2019 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Loader for event file data for an entire TensorBoard log directory."""


import collections
import os

from tensorboard.backend.event_processing import directory_watcher
from tensorboard.backend.event_processing import io_wrapper
from tensorboard.util import tb_logging


logger = tb_logging.get_logger()


class LogdirLoader:
    """Loader for a root log directory, maintaining multiple DirectoryLoaders.

    This class takes a root log directory and a factory for DirectoryLoaders, and
    maintains one DirectoryLoader per "logdir subdirectory" of the root logdir.

    Note that this class is not thread-safe.
    """

    def __init__(self, logdir, directory_loader_factory):
        """Constructs a new LogdirLoader.

        Args:
          logdir: The root log directory to load from.
          directory_loader_factory: A factory for creating DirectoryLoaders. The
            factory should take a path and return a DirectoryLoader.

        Raises:
          ValueError: If logdir or directory_loader_factory are None.
        """
        if logdir is None:
            raise ValueError("A logdir is required")
        if directory_loader_factory is None:
            raise ValueError("A directory loader factory is required")
        self._logdir = logdir
        self._directory_loader_factory = directory_loader_factory
        # Maps run names to corresponding DirectoryLoader instances.
        self._directory_loaders = {}

    def synchronize_runs(self):
        """Finds new runs within `logdir` and makes `DirectoryLoaders` for
        them.

        In addition, any existing `DirectoryLoader` whose run directory
        no longer exists will be deleted.
        """
        logger.info("Starting logdir traversal of %s", self._logdir)
        runs_seen = set()
        for subdir in io_wrapper.GetLogdirSubdirectories(self._logdir):
            run = os.path.relpath(subdir, self._logdir)
            runs_seen.add(run)
            if run not in self._directory_loaders:
                logger.info("- Adding run for relative directory %s", run)
                self._directory_loaders[run] = self._directory_loader_factory(
                    subdir
                )
        stale_runs = set(self._directory_loaders) - runs_seen
        if stale_runs:
            for run in stale_runs:
                logger.info("- Removing run for relative directory %s", run)
                del self._directory_loaders[run]
        logger.info("Ending logdir traversal of %s", self._logdir)

    def get_run_events(self):
        """Returns tf.Event generators for each run's `DirectoryLoader`.

        Warning: the generators are stateful and consuming them will affect the
        results of any other existing generators for that run; calling code should
        ensure it takes events from only a single generator per run at a time.

        Returns:
          Dictionary containing an entry for each run, mapping the run name to a
          generator yielding tf.Event protobuf objects loaded from that run.
        """
        runs = list(self._directory_loaders)
        logger.info("Creating event loading generators for %d runs", len(runs))
        run_to_loader = collections.OrderedDict()
        for run_name in sorted(runs):
            loader = self._directory_loaders[run_name]
            run_to_loader[run_name] = self._wrap_loader_generator(loader.Load())
        return run_to_loader

    def _wrap_loader_generator(self, loader_generator):
        """Wraps `DirectoryLoader` generator to swallow
        `DirectoryDeletedError`."""
        try:
            for item in loader_generator:
                yield item
        except directory_watcher.DirectoryDeletedError:
            return



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/compat/proto/trackable_object_graph.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5tensorboard/compat/proto/trackable_object_graph.proto\x12\x0btensorboard\x1a\x1egoogle/protobuf/wrappers.proto\"\xf8\x05\n\x14TrackableObjectGraph\x12@\n\x05nodes\x18\x01 \x03(\x0b\x32\x31.tensorboard.TrackableObjectGraph.TrackableObject\x1a\x9d\x05\n\x0fTrackableObject\x12S\n\x08\x63hildren\x18\x01 \x03(\x0b\x32\x41.tensorboard.TrackableObjectGraph.TrackableObject.ObjectReference\x12V\n\nattributes\x18\x02 \x03(\x0b\x32\x42.tensorboard.TrackableObjectGraph.TrackableObject.SerializedTensor\x12_\n\x0eslot_variables\x18\x03 \x03(\x0b\x32G.tensorboard.TrackableObjectGraph.TrackableObject.SlotVariableReference\x12\x36\n\x10registered_saver\x18\x04 \x01(\x0b\x32\x1c.tensorboard.RegisteredSaver\x12\x39\n\x15has_checkpoint_values\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x1a\x36\n\x0fObjectReference\x12\x0f\n\x07node_id\x18\x01 \x01(\x05\x12\x12\n\nlocal_name\x18\x02 \x01(\t\x1a\x63\n\x10SerializedTensor\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\tfull_name\x18\x02 \x01(\t\x12\x16\n\x0e\x63heckpoint_key\x18\x03 \x01(\tJ\x04\x08\x04\x10\x05R\x10optional_restore\x1al\n\x15SlotVariableReference\x12!\n\x19original_variable_node_id\x18\x01 \x01(\x05\x12\x11\n\tslot_name\x18\x02 \x01(\t\x12\x1d\n\x15slot_variable_node_id\x18\x03 \x01(\x05\"4\n\x0fRegisteredSaver\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0bobject_name\x18\x02 \x01(\tBZZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01\x62\x06proto3')



_TRACKABLEOBJECTGRAPH = DESCRIPTOR.message_types_by_name['TrackableObjectGraph']
_TRACKABLEOBJECTGRAPH_TRACKABLEOBJECT = _TRACKABLEOBJECTGRAPH.nested_types_by_name['TrackableObject']
_TRACKABLEOBJECTGRAPH_TRACKABLEOBJECT_OBJECTREFERENCE = _TRACKABLEOBJECTGRAPH_TRACKABLEOBJECT.nested_types_by_name['ObjectReference']
_TRACKABLEOBJECTGRAPH_TRACKABLEOBJECT_SERIALIZEDTENSOR = _TRACKABLEOBJECTGRAPH_TRACKABLEOBJECT.nested_types_by_name['SerializedTensor']
_TRACKABLEOBJECTGRAPH_TRACKABLEOBJECT_SLOTVARIABLEREFERENCE = _TRACKABLEOBJECTGRAPH_TRACKABLEOBJECT.nested_types_by_name['SlotVariableReference']
_REGISTEREDSAVER = DESCRIPTOR.message_types_by_name['RegisteredSaver']
TrackableObjectGraph = _reflection.GeneratedProtocolMessageType('TrackableObjectGraph', (_message.Message,), {

  'TrackableObject' : _reflection.GeneratedProtocolMessageType('TrackableObject', (_message.Message,), {

    'ObjectReference' : _reflection.GeneratedProtocolMessageType('ObjectReference', (_message.Message,), {
      'DESCRIPTOR' : _TRACKABLEOBJECTGRAPH_TRACKABLEOBJECT_OBJECTREFERENCE,
      '__module__' : 'tensorboard.compat.proto.trackable_object_graph_pb2'
      # @@protoc_insertion_point(class_scope:tensorboard.TrackableObjectGraph.TrackableObject.ObjectReference)
      })
    ,

    'SerializedTensor' : _reflection.GeneratedProtocolMessageType('SerializedTensor', (_message.Message,), {
      'DESCRIPTOR' : _TRACKABLEOBJECTGRAPH_TRACKABLEOBJECT_SERIALIZEDTENSOR,
      '__module__' : 'tensorboard.compat.proto.trackable_object_graph_pb2'
      # @@protoc_insertion_point(class_scope:tensorboard.TrackableObjectGraph.TrackableObject.SerializedTensor)
      })
    ,

    'SlotVariableReference' : _reflection.GeneratedProtocolMessageType('SlotVariableReference', (_message.Message,), {
      'DESCRIPTOR' : _TRACKABLEOBJECTGRAPH_TRACKABLEOBJECT_SLOTVARIABLEREFERENCE,
      '__module__' : 'tensorboard.compat.proto.trackable_object_graph_pb2'
      # @@protoc_insertion_point(class_scope:tensorboard.TrackableObjectGraph.TrackableObject.SlotVariableReference)
      })
    ,
    'DESCRIPTOR' : _TRACKABLEOBJECTGRAPH_TRACKABLEOBJECT,
    '__module__' : 'tensorboard.compat.proto.trackable_object_graph_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.TrackableObjectGraph.TrackableObject)
    })
  ,
  'DESCRIPTOR' : _TRACKABLEOBJECTGRAPH,
  '__module__' : 'tensorboard.compat.proto.trackable_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.TrackableObjectGraph)
  })
_sym_db.RegisterMessage(TrackableObjectGraph)
_sym_db.RegisterMessage(TrackableObjectGraph.TrackableObject)
_sym_db.RegisterMessage(TrackableObjectGraph.TrackableObject.ObjectReference)
_sym_db.RegisterMessage(TrackableObjectGraph.TrackableObject.SerializedTensor)
_sym_db.RegisterMessage(TrackableObjectGraph.TrackableObject.SlotVariableReference)

RegisteredSaver = _reflection.GeneratedProtocolMessageType('RegisteredSaver', (_message.Message,), {
  'DESCRIPTOR' : _REGISTEREDSAVER,
  '__module__' : 'tensorboard.compat.proto.trackable_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.RegisteredSaver)
  })
_sym_db.RegisterMessage(RegisteredSaver)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\370\001\001'
  _TRACKABLEOBJECTGRAPH._serialized_start=103
  _TRACKABLEOBJECTGRAPH._serialized_end=863
  _TRACKABLEOBJECTGRAPH_TRACKABLEOBJECT._serialized_start=194
  _TRACKABLEOBJECTGRAPH_TRACKABLEOBJECT._serialized_end=863
  _TRACKABLEOBJECTGRAPH_TRACKABLEOBJECT_OBJECTREFERENCE._serialized_start=598
  _TRACKABLEOBJECTGRAPH_TRACKABLEOBJECT_OBJECTREFERENCE._serialized_end=652
  _TRACKABLEOBJECTGRAPH_TRACKABLEOBJECT_SERIALIZEDTENSOR._serialized_start=654
  _TRACKABLEOBJECTGRAPH_TRACKABLEOBJECT_SERIALIZEDTENSOR._serialized_end=753
  _TRACKABLEOBJECTGRAPH_TRACKABLEOBJECT_SLOTVARIABLEREFERENCE._serialized_start=755
  _TRACKABLEOBJECTGRAPH_TRACKABLEOBJECT_SLOTVARIABLEREFERENCE._serialized_end=863
  _REGISTEREDSAVER._serialized_start=865
  _REGISTEREDSAVER._serialized_end=917
# @@protoc_insertion_point(module_scope)



# Copyright 2020 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Experiment Data Access API for tensorboard.dev."""


import sys
import time

import grpc

from tensorboard.data.experimental import base_experiment
from tensorboard.data.experimental import utils as experimental_utils
from tensorboard.uploader import auth
from tensorboard.uploader import util
from tensorboard.uploader import server_info as server_info_lib
from tensorboard.uploader.proto import export_service_pb2
from tensorboard.uploader.proto import export_service_pb2_grpc
from tensorboard.uploader.proto import server_info_pb2
from tensorboard.util import grpc_util


DEFAULT_ORIGIN = "https://tensorboard.dev"


def import_pandas():
    """Import pandas, guarded by a user-friendly error message on failure."""
    try:
        import pandas
    except ImportError:
        raise ImportError(
            "The get_scalars() feature requires the pandas package, "
            "which does not seem to be available in your Python "
            "environment. You can install it with command:\n\n"
            "  pip install pandas\n"
        )
    return pandas


class ExperimentFromDev(base_experiment.BaseExperiment):
    """Implementation of BaseExperiment, specialized for tensorboard.dev."""

    def __init__(self, experiment_id, api_endpoint=None):
        """Constructor of ExperimentFromDev.

        Args:
          experiment_id: String ID of the experiment on tensorboard.dev (e.g.,
            "AdYd1TgeTlaLWXx6I8JUbA").
          api_endpoint: Optional override value for API endpoint. Used for
            development only.
        """
        super().__init__()
        self._experiment_id = experiment_id
        self._api_client = get_api_client(api_endpoint=api_endpoint)

    def get_scalars(
        self,
        runs_filter=None,
        tags_filter=None,
        pivot=False,
        include_wall_time=False,
    ):
        # NOTE(#3650): Import pandas early in this method, so if the
        # Python environment does not have pandas installed, an error can be
        # raised early, before any rpc call is made.
        pandas = import_pandas()
        if runs_filter is not None:
            raise NotImplementedError(
                "runs_filter support for get_scalars() is not implemented yet."
            )
        if tags_filter is not None:
            raise NotImplementedError(
                "tags_filter support for get_scalars() is not implemented yet."
            )

        request = export_service_pb2.StreamExperimentDataRequest()
        request.experiment_id = self._experiment_id
        read_time = time.time()
        util.set_timestamp(request.read_timestamp, read_time)
        # TODO(cais, wchargin): Use another rpc to check for staleness and avoid
        # a new StreamExperimentData rpc request if data is not stale.
        stream = self._api_client.StreamExperimentData(
            request, metadata=grpc_util.version_metadata()
        )

        runs = []
        tags = []
        steps = []
        wall_times = []
        values = []
        for response in stream:
            # TODO(cais, wchargin): Display progress bar during data loading.
            num_values = len(response.points.values)
            runs.extend([response.run_name] * num_values)
            tags.extend([response.tag_name] * num_values)
            steps.extend(list(response.points.steps))
            wall_times.extend(
                [t.ToNanoseconds() / 1e9 for t in response.points.wall_times]
            )
            values.extend(list(response.points.values))

        data = {
            "run": runs,
            "tag": tags,
            "step": steps,
            "value": values,
        }
        if include_wall_time:
            data["wall_time"] = wall_times
        dataframe = pandas.DataFrame(data)
        if pivot:
            dataframe = experimental_utils.pivot_dataframe(dataframe)
        return dataframe


def get_api_client(api_endpoint=None):
    server_info = _get_server_info(api_endpoint=api_endpoint)
    _handle_server_info(server_info)
    channel_creds = grpc.ssl_channel_credentials()
    credentials = auth.CredentialsStore().read_credentials()
    if credentials:
        channel_creds = grpc.composite_channel_credentials(
            channel_creds, auth.id_token_call_credentials(credentials)
        )
    channel = grpc.secure_channel(
        server_info.api_server.endpoint, channel_creds
    )
    return export_service_pb2_grpc.TensorBoardExporterServiceStub(channel)


def _get_server_info(api_endpoint=None):
    # TODO(cais): Add more plugins to the list when more plugin/data types
    # are supported
    plugins = ["scalars"]
    if api_endpoint:
        return server_info_lib.create_server_info(
            DEFAULT_ORIGIN, api_endpoint, plugins
        )
    return server_info_lib.fetch_server_info(DEFAULT_ORIGIN, plugins)


def _handle_server_info(info):
    compat = info.compatibility
    if compat.verdict == server_info_pb2.VERDICT_WARN:
        sys.stderr.write("Warning [from server]: %s\n" % compat.details)
        sys.stderr.flush()
    elif compat.verdict == server_info_pb2.VERDICT_ERROR:
        raise ValueError("Error [from server]: %s" % compat.details)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/uploader/proto/server_info.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,tensorboard/uploader/proto/server_info.proto\x12\x13tensorboard.service\"l\n\x11ServerInfoRequest\x12\x0f\n\x07version\x18\x01 \x01(\t\x12\x46\n\x14plugin_specification\x18\x02 \x01(\x0b\x32(.tensorboard.service.PluginSpecification\"\xb7\x02\n\x12ServerInfoResponse\x12\x39\n\rcompatibility\x18\x01 \x01(\x0b\x32\".tensorboard.service.Compatibility\x12\x32\n\napi_server\x18\x02 \x01(\x0b\x32\x1e.tensorboard.service.ApiServer\x12<\n\nurl_format\x18\x03 \x01(\x0b\x32(.tensorboard.service.ExperimentUrlFormat\x12:\n\x0eplugin_control\x18\x04 \x01(\x0b\x32\".tensorboard.service.PluginControl\x12\x38\n\rupload_limits\x18\x05 \x01(\x0b\x32!.tensorboard.service.UploadLimits\"\\\n\rCompatibility\x12:\n\x07verdict\x18\x01 \x01(\x0e\x32).tensorboard.service.CompatibilityVerdict\x12\x0f\n\x07\x64\x65tails\x18\x02 \x01(\t\"\x1d\n\tApiServer\x12\x10\n\x08\x65ndpoint\x18\x01 \x01(\t\"?\n\x13\x45xperimentUrlFormat\x12\x10\n\x08template\x18\x01 \x01(\t\x12\x16\n\x0eid_placeholder\x18\x02 \x01(\t\"-\n\x13PluginSpecification\x12\x16\n\x0eupload_plugins\x18\x02 \x03(\t\"(\n\rPluginControl\x12\x17\n\x0f\x61llowed_plugins\x18\x01 \x03(\t\"\x92\x02\n\x0cUploadLimits\x12\x1f\n\x17max_scalar_request_size\x18\x03 \x01(\x03\x12\x1f\n\x17max_tensor_request_size\x18\x04 \x01(\x03\x12\x1d\n\x15max_blob_request_size\x18\x05 \x01(\x03\x12#\n\x1bmin_scalar_request_interval\x18\x06 \x01(\x03\x12#\n\x1bmin_tensor_request_interval\x18\x07 \x01(\x03\x12!\n\x19min_blob_request_interval\x18\x08 \x01(\x03\x12\x15\n\rmax_blob_size\x18\x01 \x01(\x03\x12\x1d\n\x15max_tensor_point_size\x18\x02 \x01(\x03*`\n\x14\x43ompatibilityVerdict\x12\x13\n\x0fVERDICT_UNKNOWN\x10\x00\x12\x0e\n\nVERDICT_OK\x10\x01\x12\x10\n\x0cVERDICT_WARN\x10\x02\x12\x11\n\rVERDICT_ERROR\x10\x03\x62\x06proto3')

_COMPATIBILITYVERDICT = DESCRIPTOR.enum_types_by_name['CompatibilityVerdict']
CompatibilityVerdict = enum_type_wrapper.EnumTypeWrapper(_COMPATIBILITYVERDICT)
VERDICT_UNKNOWN = 0
VERDICT_OK = 1
VERDICT_WARN = 2
VERDICT_ERROR = 3


_SERVERINFOREQUEST = DESCRIPTOR.message_types_by_name['ServerInfoRequest']
_SERVERINFORESPONSE = DESCRIPTOR.message_types_by_name['ServerInfoResponse']
_COMPATIBILITY = DESCRIPTOR.message_types_by_name['Compatibility']
_APISERVER = DESCRIPTOR.message_types_by_name['ApiServer']
_EXPERIMENTURLFORMAT = DESCRIPTOR.message_types_by_name['ExperimentUrlFormat']
_PLUGINSPECIFICATION = DESCRIPTOR.message_types_by_name['PluginSpecification']
_PLUGINCONTROL = DESCRIPTOR.message_types_by_name['PluginControl']
_UPLOADLIMITS = DESCRIPTOR.message_types_by_name['UploadLimits']
ServerInfoRequest = _reflection.GeneratedProtocolMessageType('ServerInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _SERVERINFOREQUEST,
  '__module__' : 'tensorboard.uploader.proto.server_info_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.ServerInfoRequest)
  })
_sym_db.RegisterMessage(ServerInfoRequest)

ServerInfoResponse = _reflection.GeneratedProtocolMessageType('ServerInfoResponse', (_message.Message,), {
  'DESCRIPTOR' : _SERVERINFORESPONSE,
  '__module__' : 'tensorboard.uploader.proto.server_info_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.ServerInfoResponse)
  })
_sym_db.RegisterMessage(ServerInfoResponse)

Compatibility = _reflection.GeneratedProtocolMessageType('Compatibility', (_message.Message,), {
  'DESCRIPTOR' : _COMPATIBILITY,
  '__module__' : 'tensorboard.uploader.proto.server_info_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.Compatibility)
  })
_sym_db.RegisterMessage(Compatibility)

ApiServer = _reflection.GeneratedProtocolMessageType('ApiServer', (_message.Message,), {
  'DESCRIPTOR' : _APISERVER,
  '__module__' : 'tensorboard.uploader.proto.server_info_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.ApiServer)
  })
_sym_db.RegisterMessage(ApiServer)

ExperimentUrlFormat = _reflection.GeneratedProtocolMessageType('ExperimentUrlFormat', (_message.Message,), {
  'DESCRIPTOR' : _EXPERIMENTURLFORMAT,
  '__module__' : 'tensorboard.uploader.proto.server_info_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.ExperimentUrlFormat)
  })
_sym_db.RegisterMessage(ExperimentUrlFormat)

PluginSpecification = _reflection.GeneratedProtocolMessageType('PluginSpecification', (_message.Message,), {
  'DESCRIPTOR' : _PLUGINSPECIFICATION,
  '__module__' : 'tensorboard.uploader.proto.server_info_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.PluginSpecification)
  })
_sym_db.RegisterMessage(PluginSpecification)

PluginControl = _reflection.GeneratedProtocolMessageType('PluginControl', (_message.Message,), {
  'DESCRIPTOR' : _PLUGINCONTROL,
  '__module__' : 'tensorboard.uploader.proto.server_info_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.PluginControl)
  })
_sym_db.RegisterMessage(PluginControl)

UploadLimits = _reflection.GeneratedProtocolMessageType('UploadLimits', (_message.Message,), {
  'DESCRIPTOR' : _UPLOADLIMITS,
  '__module__' : 'tensorboard.uploader.proto.server_info_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.UploadLimits)
  })
_sym_db.RegisterMessage(UploadLimits)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _COMPATIBILITYVERDICT._serialized_start=1049
  _COMPATIBILITYVERDICT._serialized_end=1145
  _SERVERINFOREQUEST._serialized_start=69
  _SERVERINFOREQUEST._serialized_end=177
  _SERVERINFORESPONSE._serialized_start=180
  _SERVERINFORESPONSE._serialized_end=491
  _COMPATIBILITY._serialized_start=493
  _COMPATIBILITY._serialized_end=585
  _APISERVER._serialized_start=587
  _APISERVER._serialized_end=616
  _EXPERIMENTURLFORMAT._serialized_start=618
  _EXPERIMENTURLFORMAT._serialized_end=681
  _PLUGINSPECIFICATION._serialized_start=683
  _PLUGINSPECIFICATION._serialized_end=728
  _PLUGINCONTROL._serialized_start=730
  _PLUGINCONTROL._serialized_end=770
  _UPLOADLIMITS._serialized_start=773
  _UPLOADLIMITS._serialized_end=1047
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/compat/proto/tensor_shape.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+tensorboard/compat/proto/tensor_shape.proto\x12\x0btensorboard\"{\n\x10TensorShapeProto\x12.\n\x03\x64im\x18\x02 \x03(\x0b\x32!.tensorboard.TensorShapeProto.Dim\x12\x14\n\x0cunknown_rank\x18\x03 \x01(\x08\x1a!\n\x03\x44im\x12\x0c\n\x04size\x18\x01 \x01(\x03\x12\x0c\n\x04name\x18\x02 \x01(\tB\x87\x01\n\x18org.tensorflow.frameworkB\x11TensorShapeProtosP\x01ZSgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/tensor_shape_go_proto\xf8\x01\x01\x62\x06proto3')



_TENSORSHAPEPROTO = DESCRIPTOR.message_types_by_name['TensorShapeProto']
_TENSORSHAPEPROTO_DIM = _TENSORSHAPEPROTO.nested_types_by_name['Dim']
TensorShapeProto = _reflection.GeneratedProtocolMessageType('TensorShapeProto', (_message.Message,), {

  'Dim' : _reflection.GeneratedProtocolMessageType('Dim', (_message.Message,), {
    'DESCRIPTOR' : _TENSORSHAPEPROTO_DIM,
    '__module__' : 'tensorboard.compat.proto.tensor_shape_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.TensorShapeProto.Dim)
    })
  ,
  'DESCRIPTOR' : _TENSORSHAPEPROTO,
  '__module__' : 'tensorboard.compat.proto.tensor_shape_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.TensorShapeProto)
  })
_sym_db.RegisterMessage(TensorShapeProto)
_sym_db.RegisterMessage(TensorShapeProto.Dim)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030org.tensorflow.frameworkB\021TensorShapeProtosP\001ZSgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/tensor_shape_go_proto\370\001\001'
  _TENSORSHAPEPROTO._serialized_start=60
  _TENSORSHAPEPROTO._serialized_end=183
  _TENSORSHAPEPROTO_DIM._serialized_start=150
  _TENSORSHAPEPROTO_DIM._serialized_end=183
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/compat/proto/cost_graph.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorboard.compat.proto import tensor_shape_pb2 as tensorboard_dot_compat_dot_proto_dot_tensor__shape__pb2
from tensorboard.compat.proto import types_pb2 as tensorboard_dot_compat_dot_proto_dot_types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n)tensorboard/compat/proto/cost_graph.proto\x12\x0btensorboard\x1a+tensorboard/compat/proto/tensor_shape.proto\x1a$tensorboard/compat/proto/types.proto\"\xd0\x06\n\x0c\x43ostGraphDef\x12,\n\x04node\x18\x01 \x03(\x0b\x32\x1e.tensorboard.CostGraphDef.Node\x12\x36\n\x04\x63ost\x18\x02 \x03(\x0b\x32(.tensorboard.CostGraphDef.AggregatedCost\x1a\xa6\x05\n\x04Node\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0e\n\x06\x64\x65vice\x18\x02 \x01(\t\x12\n\n\x02id\x18\x03 \x01(\x05\x12<\n\ninput_info\x18\x04 \x03(\x0b\x32(.tensorboard.CostGraphDef.Node.InputInfo\x12>\n\x0boutput_info\x18\x05 \x03(\x0b\x32).tensorboard.CostGraphDef.Node.OutputInfo\x12\x1d\n\x15temporary_memory_size\x18\x06 \x01(\x03\x12\x1e\n\x16persistent_memory_size\x18\x0c \x01(\x03\x12!\n\x15host_temp_memory_size\x18\n \x01(\x03\x42\x02\x18\x01\x12#\n\x17\x64\x65vice_temp_memory_size\x18\x0b \x01(\x03\x42\x02\x18\x01\x12)\n\x1d\x64\x65vice_persistent_memory_size\x18\x10 \x01(\x03\x42\x02\x18\x01\x12\x14\n\x0c\x63ompute_cost\x18\t \x01(\x03\x12\x14\n\x0c\x63ompute_time\x18\x0e \x01(\x03\x12\x13\n\x0bmemory_time\x18\x0f \x01(\x03\x12\x10\n\x08is_final\x18\x07 \x01(\x08\x12\x15\n\rcontrol_input\x18\x08 \x03(\x05\x12\x12\n\ninaccurate\x18\x11 \x01(\x08\x1a;\n\tInputInfo\x12\x16\n\x0epreceding_node\x18\x01 \x01(\x05\x12\x16\n\x0epreceding_port\x18\x02 \x01(\x05\x1a\x88\x01\n\nOutputInfo\x12\x0c\n\x04size\x18\x01 \x01(\x03\x12\x18\n\x10\x61lias_input_port\x18\x02 \x01(\x03\x12,\n\x05shape\x18\x03 \x01(\x0b\x32\x1d.tensorboard.TensorShapeProto\x12$\n\x05\x64type\x18\x04 \x01(\x0e\x32\x15.tensorboard.DataType\x1a\x31\n\x0e\x41ggregatedCost\x12\x0c\n\x04\x63ost\x18\x01 \x01(\x02\x12\x11\n\tdimension\x18\x02 \x01(\tB\x83\x01\n\x18org.tensorflow.frameworkB\x0f\x43ostGraphProtosP\x01ZQgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/cost_graph_go_proto\xf8\x01\x01\x62\x06proto3')



_COSTGRAPHDEF = DESCRIPTOR.message_types_by_name['CostGraphDef']
_COSTGRAPHDEF_NODE = _COSTGRAPHDEF.nested_types_by_name['Node']
_COSTGRAPHDEF_NODE_INPUTINFO = _COSTGRAPHDEF_NODE.nested_types_by_name['InputInfo']
_COSTGRAPHDEF_NODE_OUTPUTINFO = _COSTGRAPHDEF_NODE.nested_types_by_name['OutputInfo']
_COSTGRAPHDEF_AGGREGATEDCOST = _COSTGRAPHDEF.nested_types_by_name['AggregatedCost']
CostGraphDef = _reflection.GeneratedProtocolMessageType('CostGraphDef', (_message.Message,), {

  'Node' : _reflection.GeneratedProtocolMessageType('Node', (_message.Message,), {

    'InputInfo' : _reflection.GeneratedProtocolMessageType('InputInfo', (_message.Message,), {
      'DESCRIPTOR' : _COSTGRAPHDEF_NODE_INPUTINFO,
      '__module__' : 'tensorboard.compat.proto.cost_graph_pb2'
      # @@protoc_insertion_point(class_scope:tensorboard.CostGraphDef.Node.InputInfo)
      })
    ,

    'OutputInfo' : _reflection.GeneratedProtocolMessageType('OutputInfo', (_message.Message,), {
      'DESCRIPTOR' : _COSTGRAPHDEF_NODE_OUTPUTINFO,
      '__module__' : 'tensorboard.compat.proto.cost_graph_pb2'
      # @@protoc_insertion_point(class_scope:tensorboard.CostGraphDef.Node.OutputInfo)
      })
    ,
    'DESCRIPTOR' : _COSTGRAPHDEF_NODE,
    '__module__' : 'tensorboard.compat.proto.cost_graph_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.CostGraphDef.Node)
    })
  ,

  'AggregatedCost' : _reflection.GeneratedProtocolMessageType('AggregatedCost', (_message.Message,), {
    'DESCRIPTOR' : _COSTGRAPHDEF_AGGREGATEDCOST,
    '__module__' : 'tensorboard.compat.proto.cost_graph_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.CostGraphDef.AggregatedCost)
    })
  ,
  'DESCRIPTOR' : _COSTGRAPHDEF,
  '__module__' : 'tensorboard.compat.proto.cost_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.CostGraphDef)
  })
_sym_db.RegisterMessage(CostGraphDef)
_sym_db.RegisterMessage(CostGraphDef.Node)
_sym_db.RegisterMessage(CostGraphDef.Node.InputInfo)
_sym_db.RegisterMessage(CostGraphDef.Node.OutputInfo)
_sym_db.RegisterMessage(CostGraphDef.AggregatedCost)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030org.tensorflow.frameworkB\017CostGraphProtosP\001ZQgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/cost_graph_go_proto\370\001\001'
  _COSTGRAPHDEF_NODE.fields_by_name['host_temp_memory_size']._options = None
  _COSTGRAPHDEF_NODE.fields_by_name['host_temp_memory_size']._serialized_options = b'\030\001'
  _COSTGRAPHDEF_NODE.fields_by_name['device_temp_memory_size']._options = None
  _COSTGRAPHDEF_NODE.fields_by_name['device_temp_memory_size']._serialized_options = b'\030\001'
  _COSTGRAPHDEF_NODE.fields_by_name['device_persistent_memory_size']._options = None
  _COSTGRAPHDEF_NODE.fields_by_name['device_persistent_memory_size']._serialized_options = b'\030\001'
  _COSTGRAPHDEF._serialized_start=142
  _COSTGRAPHDEF._serialized_end=990
  _COSTGRAPHDEF_NODE._serialized_start=261
  _COSTGRAPHDEF_NODE._serialized_end=939
  _COSTGRAPHDEF_NODE_INPUTINFO._serialized_start=741
  _COSTGRAPHDEF_NODE_INPUTINFO._serialized_end=800
  _COSTGRAPHDEF_NODE_OUTPUTINFO._serialized_start=803
  _COSTGRAPHDEF_NODE_OUTPUTINFO._serialized_end=939
  _COSTGRAPHDEF_AGGREGATEDCOST._serialized_start=941
  _COSTGRAPHDEF_AGGREGATEDCOST._serialized_end=990
# @@protoc_insertion_point(module_scope)
